@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Jochen Becher
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Jochen Becher
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "treemodel.h"
 
@@ -36,6 +14,7 @@
 #include "qmt/model/mitem.h"
 #include "qmt/model/mrelation.h"
 #include "qmt/model/massociation.h"
+#include "qmt/model/mconnection.h"
 #include "qmt/model/mdependency.h"
 #include "qmt/model/minheritance.h"
 
@@ -47,8 +26,12 @@
 #include "qmt/style/style.h"
 #include "qmt/style/stylecontroller.h"
 
+#include "../../modelinglibtr.h"
+
 #include <QStandardItem>
 
+using Utils::FilePath;
+
 namespace qmt {
 
 class TreeModel::ModelItem : public QStandardItem
@@ -77,120 +60,129 @@ public:
 
     ModelItem *product() const { return m_item; }
 
-    void visitMElement(const MElement *element)
+    void visitMElement(const MElement *element) final
     {
-        Q_UNUSED(element);
+        Q_UNUSED(element)
         QMT_CHECK(false);
     }
 
-    void visitMObject(const MObject *object)
+    void visitMObject(const MObject *object) final
     {
-        Q_UNUSED(object);
-        QMT_CHECK(m_item);
+        Q_UNUSED(object)
+        QMT_ASSERT(m_item, return);
         m_item->setEditable(false);
     }
 
-    void visitMPackage(const MPackage *package)
+    void visitMPackage(const MPackage *package) final
     {
         QMT_CHECK(!m_item);
 
-        static QIcon icon(QStringLiteral(":/modelinglib/48x48/package.png"));
+        static QIcon icon(":/modelinglib/48x48/package.png");
         m_item = new ModelItem(icon, m_treeModel->createObjectLabel(package));
         m_item->setData(TreeModel::Package, TreeModel::RoleItemType);
         visitMObject(package);
     }
 
-    void visitMClass(const MClass *klass)
+    void visitMClass(const MClass *klass) final
     {
         QMT_CHECK(!m_item);
 
         QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementClass, StyleEngine::TypeClass, klass->stereotypes(),
-                                             QStringLiteral(":/modelinglib/48x48/class.png"));
+                                             ":/modelinglib/48x48/class.png");
         m_item = new ModelItem(icon, m_treeModel->createObjectLabel(klass));
         m_item->setData(TreeModel::Element, TreeModel::RoleItemType);
         m_item->setStereotypes(klass->stereotypes());
         visitMObject(klass);
     }
 
-    void visitMComponent(const MComponent *component)
+    void visitMComponent(const MComponent *component) final
     {
         QMT_CHECK(!m_item);
 
         QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementComponent, StyleEngine::TypeComponent,
                                              component->stereotypes(),
-                                             QStringLiteral(":/modelinglib/48x48/component.png"));
+                                             ":/modelinglib/48x48/component.png");
         m_item = new ModelItem(icon, m_treeModel->createObjectLabel(component));
         m_item->setData(TreeModel::Element, TreeModel::RoleItemType);
         m_item->setStereotypes(component->stereotypes());
         visitMObject(component);
     }
 
-    void visitMDiagram(const MDiagram *diagram)
+    void visitMDiagram(const MDiagram *diagram) final
     {
         visitMObject(diagram);
         m_item->setData(TreeModel::Diagram, TreeModel::RoleItemType);
     }
 
-    void visitMCanvasDiagram(const MCanvasDiagram *diagram)
+    void visitMCanvasDiagram(const MCanvasDiagram *diagram) final
     {
         QMT_CHECK(!m_item);
 
-        static QIcon icon(QStringLiteral(":/modelinglib/48x48/canvas-diagram.png"));
+        static QIcon icon(":/modelinglib/48x48/canvas-diagram.png");
         m_item = new ModelItem(icon, m_treeModel->createObjectLabel(diagram));
         visitMDiagram(diagram);
     }
 
-    void visitMItem(const MItem *item)
+    void visitMItem(const MItem *item) final
     {
         QMT_CHECK(!m_item);
 
         QList<QString> stereotypes = item->stereotypes() << item->variety();
         QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementItem, StyleEngine::TypeItem, stereotypes,
-                                             QStringLiteral(":/modelinglib/48x48/item.png"));
+                                             ":/modelinglib/48x48/item.png");
         m_item = new ModelItem(icon, m_treeModel->createObjectLabel(item));
         m_item->setData(TreeModel::Element, TreeModel::RoleItemType);
         m_item->setStereotypes(stereotypes);
         visitMObject(item);
     }
 
-    void visitMRelation(const MRelation *relation)
+    void visitMRelation(const MRelation *relation) final
     {
-        Q_UNUSED(relation);
-        QMT_CHECK(m_item);
+        Q_UNUSED(relation)
+        QMT_ASSERT(m_item, return);
         m_item->setEditable(false);
         m_item->setData(TreeModel::Relation, TreeModel::RoleItemType);
     }
 
-    void visitMDependency(const MDependency *dependency)
+    void visitMDependency(const MDependency *dependency) final
     {
         QMT_CHECK(!m_item);
 
-        static QIcon icon(QStringLiteral(":/modelinglib/48x48/dependency.png"));
+        static QIcon icon(":/modelinglib/48x48/dependency.png");
         m_item = new ModelItem(icon, m_treeModel->createRelationLabel(dependency));
         visitMRelation(dependency);
     }
 
-    void visitMInheritance(const MInheritance *inheritance)
+    void visitMInheritance(const MInheritance *inheritance) final
     {
         QMT_CHECK(!m_item);
 
-        static QIcon icon(QStringLiteral(":/modelinglib/48x48/inheritance.png"));
+        static QIcon icon(":/modelinglib/48x48/inheritance.png");
         m_item = new ModelItem(icon, m_treeModel->createRelationLabel(inheritance));
         visitMRelation(inheritance);
     }
 
-    void visitMAssociation(const MAssociation *association)
+    void visitMAssociation(const MAssociation *association) final
     {
         QMT_CHECK(!m_item);
 
-        static QIcon icon(QStringLiteral(":/modelinglib/48x48/association.png"));
+        static QIcon icon(":/modelinglib/48x48/association.png");
         m_item = new ModelItem(icon, m_treeModel->createRelationLabel(association));
         visitMRelation(association);
     }
 
+    void visitMConnection(const MConnection *connection) final
+    {
+        QMT_CHECK(!m_item);
+
+        static QIcon icon(":modelinglib/48x48/connection.ong");
+        m_item = new ModelItem(icon, m_treeModel->createRelationLabel(connection));
+        visitMRelation(connection);
+    }
+
 private:
-    TreeModel *m_treeModel = 0;
-    TreeModel::ModelItem *m_item = 0;
+    TreeModel *m_treeModel = nullptr;
+    TreeModel::ModelItem *m_item = nullptr;
 };
 
 class TreeModel::ItemUpdater : public MConstVisitor
@@ -204,93 +196,98 @@ public:
         QMT_CHECK(m_item);
     }
 
-    void visitMElement(const MElement *element)
+    void visitMElement(const MElement *element) final
     {
-        Q_UNUSED(element);
+        Q_UNUSED(element)
         QMT_CHECK(false);
     }
 
-    void visitMObject(const MObject *object)
+    void visitMObject(const MObject *object) final
     {
         updateObjectLabel(object);
     }
 
-    void visitMPackage(const MPackage *package)
+    void visitMPackage(const MPackage *package) final
     {
         visitMObject(package);
     }
 
-    void visitMClass(const MClass *klass)
+    void visitMClass(const MClass *klass) final
     {
         if (klass->stereotypes() != m_item->stereotypes()) {
             QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementClass, StyleEngine::TypeClass,
-                                                 klass->stereotypes(), QStringLiteral(":/modelinglib/48x48/class.png"));
+                                                 klass->stereotypes(), ":/modelinglib/48x48/class.png");
             m_item->setIcon(icon);
             m_item->setStereotypes(klass->stereotypes());
         }
         visitMObject(klass);
     }
 
-    void visitMComponent(const MComponent *component)
+    void visitMComponent(const MComponent *component) final
     {
         if (component->stereotypes() != m_item->stereotypes()) {
             QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementComponent, StyleEngine::TypeComponent,
                                                  component->stereotypes(),
-                                                 QStringLiteral(":/modelinglib/48x48/component.png"));
+                                                 ":/modelinglib/48x48/component.png");
             m_item->setIcon(icon);
             m_item->setStereotypes(component->stereotypes());
         }
         visitMObject(component);
     }
 
-    void visitMDiagram(const MDiagram *diagram)
+    void visitMDiagram(const MDiagram *diagram) final
     {
         visitMObject(diagram);
     }
 
-    void visitMCanvasDiagram(const MCanvasDiagram *diagram)
+    void visitMCanvasDiagram(const MCanvasDiagram *diagram) final
     {
         visitMDiagram(diagram);
     }
 
-    void visitMItem(const MItem *item)
+    void visitMItem(const MItem *item) final
     {
         QList<QString> stereotypes = item->stereotypes() << item->variety();
         if (stereotypes != m_item->stereotypes()) {
             QIcon icon = m_treeModel->createIcon(StereotypeIcon::ElementItem, StyleEngine::TypeItem, stereotypes,
-                                                 QStringLiteral(":/modelinglib/48x48/item.png"));
+                                                 ":/modelinglib/48x48/item.png");
             m_item->setIcon(icon);
             m_item->setStereotypes(stereotypes);
         }
         visitMObject(item);
     }
 
-    void visitMRelation(const MRelation *relation)
+    void visitMRelation(const MRelation *relation) final
     {
         updateRelationLabel(relation);
     }
 
-    void visitMDependency(const MDependency *dependency)
+    void visitMDependency(const MDependency *dependency) final
     {
         visitMRelation(dependency);
     }
 
-    void visitMInheritance(const MInheritance *inheritance)
+    void visitMInheritance(const MInheritance *inheritance) final
     {
         visitMRelation(inheritance);
     }
 
-    void visitMAssociation(const MAssociation *association)
+    void visitMAssociation(const MAssociation *association) final
     {
         visitMRelation(association);
     }
 
+    void visitMConnection(const MConnection *connection) final
+    {
+        visitMRelation(connection);
+    }
+
 private:
     void updateObjectLabel(const MObject *object);
     void updateRelationLabel(const MRelation *relation);
 
-    TreeModel *m_treeModel = 0;
-    TreeModel::ModelItem *m_item = 0;
+    TreeModel *m_treeModel = nullptr;
+    TreeModel::ModelItem *m_item = nullptr;
 };
 
 void TreeModel::ItemUpdater::updateObjectLabel(const MObject *object)
@@ -324,7 +321,7 @@ void TreeModel::setModelController(ModelController *modelController)
 {
     if (m_modelController != modelController) {
         if (m_modelController)
-            disconnect(m_modelController, 0, this, 0);
+            disconnect(m_modelController, nullptr, this, nullptr);
         m_modelController = modelController;
         if (m_modelController) {
             connect(m_modelController, &ModelController::beginResetModel,
@@ -386,18 +383,18 @@ MElement *TreeModel::element(const QModelIndex &index) const
 {
     QMT_CHECK(index.isValid());
 
-    MElement *element = 0;
+    MElement *element = nullptr;
     QStandardItem *item = itemFromIndex(index);
     if (item) {
         if (item->parent()) {
             auto parentModelItem = dynamic_cast<ModelItem *>(item->parent());
-            QMT_CHECK(parentModelItem);
+            QMT_ASSERT(parentModelItem, return nullptr);
             const MObject *parentObject = m_itemToObjectMap.value(parentModelItem);
-            QMT_CHECK(parentObject);
+            QMT_ASSERT(parentObject, return nullptr);
             if (parentObject) {
                 if (index.row() >= 0 && index.row() < parentObject->children().size()) {
                     element = parentObject->children().at(index.row());
-                    QMT_CHECK(element);
+                    QMT_ASSERT(element, return nullptr);
                 } else if (index.row() >= parentObject->children().size()
                            && index.row() < parentObject->children().size() + parentObject->relations().size()) {
                     element = parentObject->relations().at(index.row() - parentObject->children().size());
@@ -428,7 +425,7 @@ QModelIndex TreeModel::indexOf(const MElement *element) const
             QMT_CHECK(false);
             return QModelIndex();
         }
-        QModelIndex parentIndex = indexFromItem(item);
+        const QModelIndex parentIndex = indexFromItem(item);
         int row = parentObject->children().indexOf(object);
         return QStandardItemModel::index(row, 0, parentIndex);
     } else if (auto relation = dynamic_cast<const MRelation *>(element)) {
@@ -439,7 +436,7 @@ QModelIndex TreeModel::indexOf(const MElement *element) const
             QMT_CHECK(false);
             return QModelIndex();
         }
-        QModelIndex parentIndex = indexFromItem(item);
+        const QModelIndex parentIndex = indexFromItem(item);
         int row = owner->children().size() + owner->relations().indexOf(relation);
         return QStandardItemModel::index(row, 0, parentIndex);
     }
@@ -487,8 +484,8 @@ void TreeModel::onEndResetModel()
 
 void TreeModel::onBeginUpdateObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = UpdateElement;
 }
@@ -504,25 +501,25 @@ void TreeModel::onEndUpdateObject(int row, const MObject *parent)
         parentIndex = indexFromItem(parentItem);
     }
     // reflect updated element in standard item
-    QModelIndex elementIndex = this->QStandardItemModel::index(row, 0, parentIndex);
+    const QModelIndex elementIndex = this->QStandardItemModel::index(row, 0, parentIndex);
     MElement *element = TreeModel::element(elementIndex);
     if (element) {
         auto object = dynamic_cast<MObject *>(element);
         if (object) {
             auto item = dynamic_cast<ModelItem *>(itemFromIndex(elementIndex));
-            QMT_CHECK(item);
+            QMT_ASSERT(item, return);
             ItemUpdater visitor(this, item);
             element->accept(&visitor);
         }
     }
     m_busyState = NotBusy;
-    emit dataChanged(QStandardItemModel::index(row, 0, parentIndex), QStandardItemModel::index(row, 0, parentIndex));
+    emit dataChanged(elementIndex, elementIndex);
 }
 
 void TreeModel::onBeginInsertObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = InsertElement;
 }
@@ -531,7 +528,7 @@ void TreeModel::onEndInsertObject(int row, const MObject *parent)
 {
     QMT_CHECK(m_busyState == InsertElement);
     ModelItem *parentItem =m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     MObject *object = parent->children().at(row);
     ModelItem *item = createItem(object);
     parentItem->insertRow(row, item);
@@ -542,20 +539,20 @@ void TreeModel::onEndInsertObject(int row, const MObject *parent)
 void TreeModel::onBeginRemoveObject(int row, const MObject *parent)
 {
     QMT_CHECK(m_busyState == NotBusy);
+    QMT_ASSERT(parent, return);
     m_busyState = RemoveElement;
-    QMT_CHECK(parent);
     MObject *object = parent->children().at(row);
     if (object)
         removeObjectFromItemMap(object);
     ModelItem *parentItem = m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     parentItem->removeRow(row);
 }
 
 void TreeModel::onEndRemoveObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == RemoveElement);
     m_busyState = NotBusy;
 }
@@ -563,13 +560,13 @@ void TreeModel::onEndRemoveObject(int row, const MObject *parent)
 void TreeModel::onBeginMoveObject(int formerRow, const MObject *formerOwner)
 {
     QMT_CHECK(m_busyState == NotBusy);
+    QMT_ASSERT(formerOwner, return);
     m_busyState = MoveElement;
-    QMT_CHECK(formerOwner);
     MObject *object = formerOwner->children().at(formerRow);
     if (object)
         removeObjectFromItemMap(object);
     ModelItem *parentItem = m_objectToItemMap.value(formerOwner);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     parentItem->removeRow(formerRow);
 }
 
@@ -577,7 +574,7 @@ void TreeModel::onEndMoveObject(int row, const MObject *owner)
 {
     QMT_CHECK(m_busyState == MoveElement);
     ModelItem *parentItem =m_objectToItemMap.value(owner);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     MObject *object = owner->children().at(row);
     ModelItem *item = createItem(object);
     parentItem->insertRow(row, item);
@@ -587,53 +584,53 @@ void TreeModel::onEndMoveObject(int row, const MObject *owner)
 
 void TreeModel::onBeginUpdateRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = UpdateRelation;
 }
 
 void TreeModel::onEndUpdateRelation(int row, const MObject *parent)
 {
-    QMT_CHECK(parent);
+    QMT_ASSERT(parent, return);
     QMT_CHECK(m_busyState == UpdateRelation);
 
     QMT_CHECK(m_objectToItemMap.contains(parent));
     ModelItem *parentItem = m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
-    QModelIndex parentIndex = indexFromItem(parentItem);
+    QMT_ASSERT(parentItem, return);
+    const QModelIndex parentIndex = indexFromItem(parentItem);
 
     // reflect updated relation in standard item
     row += parent->children().size();
-    QModelIndex elementIndex = QStandardItemModel::index(row, 0, parentIndex);
+    const QModelIndex elementIndex = QStandardItemModel::index(row, 0, parentIndex);
     MElement *element = TreeModel::element(elementIndex);
     if (element) {
         auto relation = dynamic_cast<MRelation *>(element);
         if (relation) {
             auto item = dynamic_cast<ModelItem *>(itemFromIndex(elementIndex));
-            QMT_CHECK(item);
+            QMT_ASSERT(item, return);
             ItemUpdater visitor(this, item);
             element->accept(&visitor);
         }
     }
     m_busyState = NotBusy;
-    emit dataChanged(QStandardItemModel::index(row, 0, parentIndex), QStandardItemModel::index(row, 0, parentIndex));
+    emit dataChanged(elementIndex, elementIndex);
 }
 
 void TreeModel::onBeginInsertRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = InsertRelation;
 }
 
 void TreeModel::onEndInsertRelation(int row, const MObject *parent)
 {
-    QMT_CHECK(parent);
+    QMT_ASSERT(parent, return);
     QMT_CHECK(m_busyState == InsertRelation);
     ModelItem *parentItem =m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     MRelation *relation = parent->relations().at(row);
     ModelItem *item = createItem(relation);
     parentItem->insertRow(parent->children().size() + row, item);
@@ -642,19 +639,19 @@ void TreeModel::onEndInsertRelation(int row, const MObject *parent)
 
 void TreeModel::onBeginRemoveRelation(int row, const MObject *parent)
 {
-    QMT_CHECK(parent);
+    QMT_ASSERT(parent, return);
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = RemoveRelation;
     QMT_CHECK(parent->relations().at(row));
     ModelItem *parentItem = m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     parentItem->removeRow(parent->children().size() + row);
 }
 
 void TreeModel::onEndRemoveRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == RemoveRelation);
     m_busyState = NotBusy;
 }
@@ -662,20 +659,20 @@ void TreeModel::onEndRemoveRelation(int row, const MObject *parent)
 void TreeModel::onBeginMoveRelation(int formerRow, const MObject *formerOwner)
 {
     QMT_CHECK(m_busyState == NotBusy);
+    QMT_ASSERT(formerOwner, return);
     m_busyState = MoveElement;
-    QMT_CHECK(formerOwner);
     QMT_CHECK(formerOwner->relations().at(formerRow));
     ModelItem *parentItem = m_objectToItemMap.value(formerOwner);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     parentItem->removeRow(formerOwner->children().size() + formerRow);
 }
 
 void TreeModel::onEndMoveRelation(int row, const MObject *owner)
 {
-    QMT_CHECK(owner);
+    QMT_ASSERT(owner, return);
     QMT_CHECK(m_busyState == MoveElement);
     ModelItem *parentItem =m_objectToItemMap.value(owner);
-    QMT_CHECK(parentItem);
+    QMT_ASSERT(parentItem, return);
     MRelation *relation = owner->relations().at(row);
     ModelItem *item = createItem(relation);
     parentItem->insertRow(owner->children().size() + row, item);
@@ -684,34 +681,34 @@ void TreeModel::onEndMoveRelation(int row, const MObject *owner)
 
 void TreeModel::onRelationEndChanged(MRelation *relation, MObject *endObject)
 {
-    Q_UNUSED(endObject);
+    Q_UNUSED(endObject)
     QMT_CHECK(m_busyState == NotBusy);
 
     MObject *parent = relation->owner();
-    QMT_CHECK(parent);
+    QMT_ASSERT(parent, return);
     QMT_CHECK(m_objectToItemMap.contains(parent));
     ModelItem *parentItem = m_objectToItemMap.value(parent);
-    QMT_CHECK(parentItem);
-    QModelIndex parentIndex = indexFromItem(parentItem);
+    QMT_ASSERT(parentItem, return);
+    const QModelIndex parentIndex = indexFromItem(parentItem);
 
     int row = parent->children().size() + relation->owner()->relations().indexOf(relation);
-    QModelIndex elementIndex = QStandardItemModel::index(row, 0, parentIndex);
+    const QModelIndex elementIndex = QStandardItemModel::index(row, 0, parentIndex);
     QMT_CHECK(elementIndex.isValid());
 
     auto item = dynamic_cast<ModelItem *>(itemFromIndex(elementIndex));
-    QMT_CHECK(item);
+    QMT_ASSERT(item, return);
 
     QString label = createRelationLabel(relation);
     if (item->text() != label)
         item->setText(label);
 
-    emit dataChanged(QStandardItemModel::index(row, 0, parentIndex), QStandardItemModel::index(row, 0, parentIndex));
+    emit dataChanged(elementIndex, elementIndex);
 }
 
 void TreeModel::onModelDataChanged(const QModelIndex &topleft, const QModelIndex &bottomright)
 {
-    Q_UNUSED(topleft);
-    Q_UNUSED(bottomright);
+    Q_UNUSED(topleft)
+    Q_UNUSED(bottomright)
     // TODO fix editing object name in model tree
     // item->text() no longer returns a simple object name
     // classes contains namespace label
@@ -721,7 +718,7 @@ void TreeModel::onModelDataChanged(const QModelIndex &topleft, const QModelIndex
 void TreeModel::clear()
 {
     QStandardItemModel::clear();
-    m_rootItem = 0;
+    m_rootItem = nullptr;
     m_objectToItemMap.clear();
     m_itemToObjectMap.clear();
 }
@@ -740,14 +737,14 @@ void TreeModel::createChildren(const MObject *parentObject, ModelItem *parentIte
     m_objectToItemMap.insert(parentObject, parentItem);
     QMT_CHECK(!m_itemToObjectMap.contains(parentItem));
     m_itemToObjectMap.insert(parentItem, parentObject);
-    foreach (const Handle<MObject> &object, parentObject->children()) {
+    for (const Handle<MObject> &object : parentObject->children()) {
         if (object.hasTarget()) {
             ModelItem *item = createItem(object.target());
             parentItem->appendRow(item);
             createChildren(object.target(), item);
         }
     }
-    foreach (const Handle<MRelation> &handle, parentObject->relations()) {
+    for (const Handle<MRelation> &handle : parentObject->relations()) {
         if (handle.hasTarget()) {
             MRelation *relation = handle.target();
             ModelItem *item = createItem(relation);
@@ -758,36 +755,42 @@ void TreeModel::createChildren(const MObject *parentObject, ModelItem *parentIte
 
 void TreeModel::removeObjectFromItemMap(const MObject *object)
 {
-    QMT_CHECK(object);
+    QMT_ASSERT(object, return);
     QMT_CHECK(m_objectToItemMap.contains(object));
     ModelItem *item = m_objectToItemMap.value(object);
-    QMT_CHECK(item);
+    QMT_ASSERT(item, return);
     QMT_CHECK(m_itemToObjectMap.contains(item));
     m_itemToObjectMap.remove(item);
     m_objectToItemMap.remove(object);
-    foreach (const Handle<MObject> &child, object->children()) {
+    for (const Handle<MObject> &child : object->children()) {
         if (child.hasTarget())
             removeObjectFromItemMap(child.target());
     }
 }
 
+QString TreeModel::filterLabel(const QString &label) const
+{
+    QString s = label;
+    return s.replace("\n"," ");
+}
+
 QString TreeModel::createObjectLabel(const MObject *object)
 {
-    QMT_CHECK(object);
+    QMT_ASSERT(object, return QString());
 
     if (object->name().isEmpty()) {
         if (auto item = dynamic_cast<const MItem *>(object)) {
             if (!item->variety().isEmpty())
-                return QString(QStringLiteral("[%1]")).arg(item->variety());
+                return filterLabel(QString("[%1]").arg(item->variety()));
         }
-        return tr("[unnamed]");
+        return Tr::tr("[unnamed]");
     }
 
     if (auto klass = dynamic_cast<const MClass *>(object)) {
         if (!klass->umlNamespace().isEmpty())
-            return QString(QStringLiteral("%1 [%2]")).arg(klass->name()).arg(klass->umlNamespace());
+            return filterLabel(QString("%1 [%2]").arg(klass->name()).arg(klass->umlNamespace()));
     }
-    return object->name();
+    return filterLabel(object->name());
 }
 
 QString TreeModel::createRelationLabel(const MRelation *relation)
@@ -795,22 +798,22 @@ QString TreeModel::createRelationLabel(const MRelation *relation)
     QString name;
     if (!relation->name().isEmpty()) {
         name += relation->name();
-        name += QStringLiteral(": ");
+        name += ": ";
     }
     if (MObject *endA = m_modelController->findObject(relation->endAUid()))
         name += createObjectLabel(endA);
-    name += QStringLiteral(" - ");
+    name += " - ";
     if (MObject *endB = m_modelController->findObject(relation->endBUid()))
         name += createObjectLabel(endB);
-    return name;
+    return filterLabel(name);
 }
 
 QIcon TreeModel::createIcon(StereotypeIcon::Element stereotypeIconElement, StyleEngine::ElementType styleElementType,
-                            const QStringList &stereotypes, const QString &defaultIconPath)
+                            const QStringList &stereotypes, const FilePath &defaultIconPath)
 {
     const Style *style = m_styleController->adaptStyle(styleElementType);
     return m_stereotypeController->createIcon(stereotypeIconElement, stereotypes, defaultIconPath, style,
-                                              QSize(48, 48), QMarginsF(3.0, 2.0, 3.0, 4.0));
+                                              QSize(48, 48), QMarginsF(3.0, 2.0, 3.0, 4.0), 3.0);
 }
 
 } // namespace qmt
