@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "outputformatter.h"
 
@@ -30,11 +8,11 @@
 #include "fileinprojectfinder.h"
 #include "link.h"
 #include "qtcassert.h"
-#include "qtcprocess.h"
+#include "stringutils.h"
+#include "stylehelper.h"
 #include "theme/theme.h"
 
 #include <QDir>
-#include <QFileInfo>
 #include <QPair>
 #include <QPlainTextEdit>
 #include <QPointer>
@@ -64,7 +42,7 @@ Q_GLOBAL_STATIC_WITH_ARGS(QString, linkSep, {"::"})
 
 QString OutputLineParser::createLinkTarget(const FilePath &filePath, int line = -1, int column = -1)
 {
-    return *linkPrefix() + filePath.toString() + *linkSep() + QString::number(line)
+    return *linkPrefix() + filePath.toUrlishString() + *linkSep() + QString::number(line)
             + *linkSep() + QString::number(column);
 }
 
@@ -80,7 +58,7 @@ Link OutputLineParser::parseLinkTarget(const QString &target)
         return {};
     return Link(FilePath::fromString(parts.first()),
                 parts.length() > 1 ? parts.at(1).toInt() : 0,
-                parts.length() > 2 ? parts.at(2).toInt() : 0);
+                parts.length() > 2 ? parts.at(2).toInt() - 1 : 0);
 }
 
 // The redirection mechanism is needed for broken build tools (e.g. xcodebuild) that get invoked
@@ -138,13 +116,13 @@ FilePath OutputLineParser::absoluteFilePath(const FilePath &filePath) const
 {
     if (filePath.isEmpty())
         return filePath;
-    if (filePath.toFileInfo().isAbsolute())
+    if (filePath.isAbsolutePath())
         return filePath.cleanPath();
     FilePaths candidates;
     for (const FilePath &dir : searchDirectories()) {
-        FilePath candidate = dir.pathAppended(filePath.toString());
+        FilePath candidate = dir.resolvePath(filePath);
         if (candidate.exists() || d->skipFileExistsCheck) {
-            candidate = FilePath::fromString(QDir::cleanPath(candidate.toString()));
+            candidate = candidate.cleanPath();
             if (!candidates.contains(candidate))
                 candidates << candidate;
         }
@@ -152,7 +130,7 @@ FilePath OutputLineParser::absoluteFilePath(const FilePath &filePath) const
     if (candidates.count() == 1)
         return candidates.first();
 
-    QString fp = filePath.toString();
+    QString fp = filePath.toUrlishString();
     while (fp.startsWith("../"))
         fp.remove(0, 3);
     bool found = false;
@@ -163,26 +141,48 @@ FilePath OutputLineParser::absoluteFilePath(const FilePath &filePath) const
     return filePath;
 }
 
-void OutputLineParser::addLinkSpecForAbsoluteFilePath(OutputLineParser::LinkSpecs &linkSpecs,
-        const FilePath &filePath, int lineNo, int pos, int len)
+void OutputLineParser::addLinkSpecForAbsoluteFilePath(
+    OutputLineParser::LinkSpecs &linkSpecs,
+    const FilePath &filePath,
+    int lineNo,
+    int column,
+    int pos,
+    int len)
 {
     if (filePath.toFileInfo().isAbsolute())
-        linkSpecs.append({pos, len, createLinkTarget(filePath, lineNo)});
+        linkSpecs.append({pos, len, createLinkTarget(filePath, lineNo, column)});
 }
 
-void OutputLineParser::addLinkSpecForAbsoluteFilePath(OutputLineParser::LinkSpecs &linkSpecs,
-        const FilePath &filePath, int lineNo, const QRegularExpressionMatch &match,
-        int capIndex)
+void OutputLineParser::addLinkSpecForAbsoluteFilePath(
+    OutputLineParser::LinkSpecs &linkSpecs,
+    const FilePath &filePath,
+    int lineNo,
+    int column,
+    const QRegularExpressionMatch &match,
+    int capIndex)
 {
-    addLinkSpecForAbsoluteFilePath(linkSpecs, filePath, lineNo, match.capturedStart(capIndex),
-                                   match.capturedLength(capIndex));
+    addLinkSpecForAbsoluteFilePath(linkSpecs, filePath, lineNo, column,
+                                   match.capturedStart(capIndex), match.capturedLength(capIndex));
 }
-void OutputLineParser::addLinkSpecForAbsoluteFilePath(OutputLineParser::LinkSpecs &linkSpecs,
-        const FilePath &filePath, int lineNo, const QRegularExpressionMatch &match,
-                                                      const QString &capName)
+void OutputLineParser::addLinkSpecForAbsoluteFilePath(
+    OutputLineParser::LinkSpecs &linkSpecs,
+    const FilePath &filePath,
+    int lineNo,
+    int column,
+    const QRegularExpressionMatch &match,
+    const QString &capName)
 {
-    addLinkSpecForAbsoluteFilePath(linkSpecs, filePath, lineNo, match.capturedStart(capName),
-                                   match.capturedLength(capName));
+    addLinkSpecForAbsoluteFilePath(linkSpecs, filePath, lineNo, column,
+                                   match.capturedStart(capName), match.capturedLength(capName));
+}
+
+bool Utils::OutputLineParser::fileExists(const FilePath &fp) const
+{
+#ifdef WITH_TESTS
+    if (d->skipFileExistsCheck)
+        return !fp.isEmpty();
+#endif
+    return fp.exists();
 }
 
 QString OutputLineParser::rightTrimmed(const QString &in)
@@ -211,14 +211,13 @@ public:
     QTextCursor cursor;
     AnsiEscapeCodeHandler escapeCodeHandler;
     QPair<QString, OutputFormat> incompleteLine;
-    optional<QTextCharFormat> formatOverride;
+    std::optional<QTextCharFormat> formatOverride;
     QList<OutputLineParser *> lineParsers;
     OutputLineParser *nextParser = nullptr;
     FileInProjectFinder fileFinder;
     PostPrintAction postPrintAction;
     bool boldFontEnabled = true;
     bool prependCarriageReturn = false;
-    bool prependLineFeed = false;
     bool forwardStdOutToStdError = false;
 };
 
@@ -254,7 +253,7 @@ void OutputFormatter::setLineParsers(const QList<OutputLineParser *> &parsers)
 
 void OutputFormatter::addLineParsers(const QList<OutputLineParser *> &parsers)
 {
-    for (OutputLineParser * const p : qAsConst(parsers))
+    for (OutputLineParser * const p : std::as_const(parsers))
         addLineParser(p);
 }
 
@@ -278,7 +277,7 @@ void OutputFormatter::setFileFinder(const FileInProjectFinder &finder)
 
 void OutputFormatter::setDemoteErrorsToWarnings(bool demote)
 {
-    for (OutputLineParser * const p : qAsConst(d->lineParsers))
+    for (OutputLineParser * const p : std::as_const(d->lineParsers))
         p->setDemoteErrorsToWarnings(demote);
 }
 
@@ -287,9 +286,23 @@ void OutputFormatter::overridePostPrintAction(const PostPrintAction &postPrintAc
     d->postPrintAction = postPrintAction;
 }
 
-void OutputFormatter::doAppendMessage(const QString &text, OutputFormat format)
+static void checkAndFineTuneColors(QTextCharFormat *format)
+{
+    QTC_ASSERT(format, return);
+    const QColor background = format->hasProperty(QTextCharFormat::BackgroundBrush)
+            ? format->background().color() : Utils::creatorColor(Theme::BackgroundColorNormal);
+    const QColor fgColor = StyleHelper::ensureReadableOn(background,
+                                                         format->foreground().color());
+    format->setForeground(fgColor);
+}
+
+void OutputFormatter::doAppendMessage(const QString &text, OutputFormat format, LineStatus lineStatus)
 {
     QTextCharFormat charFmt = charFormat(format);
+    const auto addNewlineIfApplicable = [&] {
+        if (lineStatus == LineStatus::Complete)
+            append("\n", charFmt);
+    };
 
     QList<FormattedText> formattedText = parseAnsi(text, charFmt);
     const QString cleanLine = std::accumulate(formattedText.begin(), formattedText.end(), QString(),
@@ -306,22 +319,26 @@ void OutputFormatter::doAppendMessage(const QString &text, OutputFormat format)
                 ? *res.formatOverride : outputTypeForParser(involvedParsers.last(), format);
         if (formatForParser != format && cleanLine == text && formattedText.length() == 1) {
             charFmt = charFormat(formatForParser);
+            checkAndFineTuneColors(&charFmt);
             formattedText.first().format = charFmt;
         }
     }
 
     if (res.newContent) {
-        append(res.newContent.value(), charFmt);
+        append(*res.newContent, charFmt);
+        addNewlineIfApplicable();
         return;
     }
 
     const QList<FormattedText> linkified = linkifiedText(formattedText, res.linkSpecs);
-    for (const FormattedText &output : linkified)
+    for (FormattedText output : linkified) {
+        checkAndFineTuneColors(&output.format);
         append(output.text, output.format);
-    if (linkified.isEmpty())
-        append({}, charFmt); // This might cause insertion of a newline character.
+        charFmt = output.format;
+    }
+    addNewlineIfApplicable();
 
-    for (OutputLineParser * const p : qAsConst(involvedParsers)) {
+    for (OutputLineParser * const p : std::as_const(involvedParsers)) {
         if (d->postPrintAction)
             d->postPrintAction(p);
         else
@@ -344,6 +361,7 @@ OutputLineParser::Result OutputFormatter::handleMessage(const QString &text, Out
                 = d->nextParser->handleLine(text, outputTypeForParser(d->nextParser, format));
         switch (res.status) {
         case OutputLineParser::Status::Done:
+            d->nextParser->flush();
             d->nextParser = nullptr;
             return res;
         case OutputLineParser::Status::InProgress:
@@ -354,13 +372,14 @@ OutputLineParser::Result OutputFormatter::handleMessage(const QString &text, Out
         }
     }
     QTC_CHECK(!d->nextParser);
-    for (OutputLineParser * const parser : qAsConst(d->lineParsers)) {
+    for (OutputLineParser * const parser : std::as_const(d->lineParsers)) {
         if (parser == oldNextParser) // We tried that one already.
             continue;
         const OutputLineParser::Result res
                 = parser->handleLine(text, outputTypeForParser(parser, format));
         switch (res.status) {
         case OutputLineParser::Status::Done:
+            parser->flush();
             involvedParsers << parser;
             return res;
         case OutputLineParser::Status::InProgress:
@@ -404,23 +423,32 @@ const QList<FormattedText> OutputFormatter::linkifiedText(
         }
 
         for (int nextLocalTextPos = 0; nextLocalTextPos < t.text.size(); ) {
-
-            // There are no more links in this part, so copy the rest of the text as-is.
-            if (nextLinkSpecIndex >= linkSpecs.size()) {
+            const auto copyRestOfSegmentAsIs = [&] {
                 linkified << FormattedText(t.text.mid(nextLocalTextPos), t.format);
                 totalTextLengthSoFar += t.text.length() - nextLocalTextPos;
+            };
+
+            // We are out of links.
+            if (nextLinkSpecIndex >= linkSpecs.size()) {
+                copyRestOfSegmentAsIs();
                 break;
             }
 
             const OutputLineParser::LinkSpec &linkSpec = linkSpecs.at(nextLinkSpecIndex);
             const int localLinkStartPos = linkSpec.startPos - totalPreviousTextLength;
+
+            // There are more links, but not in this segment.
+            if (localLinkStartPos >= t.text.size()) {
+                copyRestOfSegmentAsIs();
+                break;
+            }
+
             ++nextLinkSpecIndex;
 
             // We ignore links that would cross format boundaries.
             if (localLinkStartPos < nextLocalTextPos
                     || localLinkStartPos + linkSpec.length > t.text.length()) {
-                linkified << FormattedText(t.text.mid(nextLocalTextPos), t.format);
-                totalTextLengthSoFar += t.text.length() - nextLocalTextPos;
+                copyRestOfSegmentAsIs();
                 break;
             }
 
@@ -442,7 +470,6 @@ void OutputFormatter::append(const QString &text, const QTextCharFormat &format)
 {
     if (!plainTextEdit())
         return;
-    flushTrailingNewline();
     int startPos = 0;
     int crPos = -1;
     while ((crPos = text.indexOf('\r', startPos)) >= 0)  {
@@ -451,14 +478,14 @@ void OutputFormatter::append(const QString &text, const QTextCharFormat &format)
         d->cursor.movePosition(QTextCursor::StartOfBlock, QTextCursor::KeepAnchor);
         startPos = crPos + 1;
     }
-    if (startPos < text.count())
+    if (startPos < text.size())
         d->cursor.insertText(text.mid(startPos), format);
 }
 
 QTextCharFormat OutputFormatter::linkFormat(const QTextCharFormat &inputFormat, const QString &href)
 {
     QTextCharFormat result = inputFormat;
-    result.setForeground(creatorTheme()->color(Theme::TextColorLink));
+    result.setForeground(creatorColor(Theme::TextColorLink));
     result.setUnderlineStyle(QTextCharFormat::SingleUnderline);
     result.setAnchor(true);
     result.setAnchorHref(href);
@@ -493,32 +520,23 @@ void OutputFormatter::initFormats()
     if (!plainTextEdit())
         return;
 
-    Theme *theme = creatorTheme();
-    d->formats[NormalMessageFormat].setForeground(theme->color(Theme::OutputPanes_NormalMessageTextColor));
-    d->formats[ErrorMessageFormat].setForeground(theme->color(Theme::OutputPanes_ErrorMessageTextColor));
-    d->formats[LogMessageFormat].setForeground(theme->color(Theme::OutputPanes_WarningMessageTextColor));
-    d->formats[StdOutFormat].setForeground(theme->color(Theme::OutputPanes_StdOutTextColor));
-    d->formats[StdErrFormat].setForeground(theme->color(Theme::OutputPanes_StdErrTextColor));
-    d->formats[DebugFormat].setForeground(theme->color(Theme::OutputPanes_DebugTextColor));
-    d->formats[GeneralMessageFormat].setForeground(theme->color(Theme::OutputPanes_DebugTextColor));
+    d->formats[NormalMessageFormat].setForeground(creatorColor(Theme::OutputPanes_NormalMessageTextColor));
+    d->formats[ErrorMessageFormat].setForeground(creatorColor(Theme::OutputPanes_ErrorMessageTextColor));
+    d->formats[LogMessageFormat].setForeground(creatorColor(Theme::OutputPanes_WarningMessageTextColor));
+    d->formats[StdOutFormat].setForeground(creatorColor(Theme::OutputPanes_StdOutTextColor));
+    d->formats[StdErrFormat].setForeground(creatorColor(Theme::OutputPanes_StdErrTextColor));
+    d->formats[DebugFormat].setForeground(creatorColor(Theme::OutputPanes_DebugTextColor));
+    d->formats[GeneralMessageFormat].setForeground(creatorColor(Theme::OutputPanes_DebugTextColor));
     setBoldFontEnabled(d->boldFontEnabled);
 }
 
 void OutputFormatter::flushIncompleteLine()
 {
     clearLastLine();
-    doAppendMessage(d->incompleteLine.first, d->incompleteLine.second);
+    doAppendMessage(d->incompleteLine.first, d->incompleteLine.second, LineStatus::Incomplete);
     d->incompleteLine.first.clear();
 }
 
-void Utils::OutputFormatter::flushTrailingNewline()
-{
-    if (d->prependLineFeed) {
-        d->cursor.insertText("\n");
-        d->prependLineFeed = false;
-    }
-}
-
 void OutputFormatter::dumpIncompleteLine(const QString &line, OutputFormat format)
 {
     if (line.isEmpty())
@@ -546,7 +564,7 @@ void OutputFormatter::handleLink(const QString &href)
     // to the line parsers.
     if (handleFileLink(href))
         return;
-    for (OutputLineParser * const f : qAsConst(d->lineParsers)) {
+    for (OutputLineParser * const f : std::as_const(d->lineParsers)) {
         if (f->handleLink(href))
             return;
     }
@@ -587,9 +605,8 @@ void OutputFormatter::flush()
 {
     if (!d->incompleteLine.first.isEmpty())
         flushIncompleteLine();
-    flushTrailingNewline();
     d->escapeCodeHandler.endFormatScope();
-    for (OutputLineParser * const p : qAsConst(d->lineParsers))
+    for (OutputLineParser * const p : std::as_const(d->lineParsers))
         p->flush();
     if (d->nextParser)
         d->nextParser->runPostPrintActions(plainTextEdit());
@@ -604,13 +621,13 @@ bool OutputFormatter::hasFatalErrors() const
 
 void OutputFormatter::addSearchDir(const FilePath &dir)
 {
-    for (OutputLineParser * const p : qAsConst(d->lineParsers))
+    for (OutputLineParser * const p : std::as_const(d->lineParsers))
         p->addSearchDir(dir);
 }
 
 void OutputFormatter::dropSearchDir(const FilePath &dir)
 {
-    for (OutputLineParser * const p : qAsConst(d->lineParsers))
+    for (OutputLineParser * const p : std::as_const(d->lineParsers))
         p->dropSearchDir(dir);
 }
 
@@ -638,7 +655,7 @@ void OutputFormatter::appendMessage(const QString &text, OutputFormat format)
         d->prependCarriageReturn = false;
         out.prepend('\r');
     }
-    out = QtcProcess::normalizeNewlines(out);
+    out = Utils::normalizeNewlines(out);
     if (out.endsWith('\r')) {
         d->prependCarriageReturn = true;
         out.chop(1);
@@ -669,8 +686,7 @@ void OutputFormatter::appendMessage(const QString &text, OutputFormat format)
             dumpIncompleteLine(out.mid(startPos), format);
             break;
         }
-        doAppendMessage(out.mid(startPos, eolPos - startPos), format);
-        d->prependLineFeed = true;
+        doAppendMessage(out.mid(startPos, eolPos - startPos), format, LineStatus::Complete);
         startPos = eolPos + 1;
     }
 }
