@@ -4,19 +4,32 @@
 #include "cmakeprocess.h"
 
 #include "builddirparameters.h"
-#include "cmakeparser.h"
+#include "cmakeautogenparser.h"
+#include "cmakeoutputparser.h"
 #include "cmakeprojectconstants.h"
 #include "cmakeprojectmanagertr.h"
 #include "cmakespecificsettings.h"
+#include "cmaketoolmanager.h"
 
+#include <coreplugin/icore.h>
 #include <coreplugin/progressmanager/processprogress.h>
+
 #include <projectexplorer/buildsystem.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/taskhub.h>
+#include <projectexplorer/kitchooser.h>
 
-#include <utils/process.h>
+#include <extensionsystem/invoker.h>
+#include <extensionsystem/pluginmanager.h>
+
+#include <utils/algorithm.h>
+#include <utils/macroexpander.h>
+#include <utils/processinfo.h>
 #include <utils/processinterface.h>
+#include <utils/qtcprocess.h>
 #include <utils/stringutils.h>
+#include <utils/stylehelper.h>
+#include <utils/theme/theme.h>
 
 using namespace Core;
 using namespace ProjectExplorer;
@@ -49,11 +62,15 @@ void CMakeProcess::run(const BuildDirParameters &parameters, const QStringList &
 
     const FilePath cmakeExecutable = cmake->cmakeExecutable();
 
+    const QString mountHint = ::CMakeProjectManager::Tr::tr(
+        "You may need to add the project directory to the list of directories that are mounted by "
+        "the build device.");
+
     if (!cmakeExecutable.ensureReachable(parameters.sourceDirectory)) {
         const QString msg = ::CMakeProjectManager::Tr::tr(
                 "The source directory %1 is not reachable by the CMake executable %2.")
             .arg(parameters.sourceDirectory.displayName()).arg(cmakeExecutable.displayName());
-        BuildSystem::appendBuildSystemOutput(msg + '\n');
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix({QString(), msg, mountHint}).join('\n'));
         emit finished(failedToStartExitCode);
         return;
     }
@@ -62,7 +79,7 @@ void CMakeProcess::run(const BuildDirParameters &parameters, const QStringList &
         const QString msg = ::CMakeProjectManager::Tr::tr(
                 "The build directory %1 is not reachable by the CMake executable %2.")
             .arg(parameters.buildDirectory.displayName()).arg(cmakeExecutable.displayName());
-        BuildSystem::appendBuildSystemOutput(msg + '\n');
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix({QString(), msg, mountHint}).join('\n'));
         emit finished(failedToStartExitCode);
         return;
     }
@@ -73,35 +90,47 @@ void CMakeProcess::run(const BuildDirParameters &parameters, const QStringList &
     if (!buildDirectory.exists()) {
         const QString msg = ::CMakeProjectManager::Tr::tr(
                 "The build directory \"%1\" does not exist").arg(buildDirectory.toUserOutput());
-        BuildSystem::appendBuildSystemOutput(msg + '\n');
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix({QString(), msg}).join('\n'));
         emit finished(failedToStartExitCode);
         return;
     }
 
-    if (buildDirectory.needsDevice()) {
-        if (cmake->cmakeExecutable().host() != buildDirectory.host()) {
+    if (!buildDirectory.isLocal()) {
+        if (!cmake->cmakeExecutable().isSameDevice(buildDirectory)) {
             const QString msg = ::CMakeProjectManager::Tr::tr(
                   "CMake executable \"%1\" and build directory \"%2\" must be on the same device.")
                     .arg(cmake->cmakeExecutable().toUserOutput(), buildDirectory.toUserOutput());
-            BuildSystem::appendBuildSystemOutput(msg + '\n');
+            BuildSystem::appendBuildSystemOutput(addCMakePrefix({QString(), msg}).join('\n'));
             emit finished(failedToStartExitCode);
             return;
         }
     }
 
     // Copy the "package-manager" CMake code from the ${IDE:ResourcePath} to the build directory
-    if (Internal::CMakeSpecificSettings::instance()->packageManagerAutoSetup.value()) {
-        const FilePath localPackageManagerDir = buildDirectory.pathAppended(Constants::PACKAGE_MANAGER_DIR);
-        const FilePath idePackageManagerDir = FilePath::fromString(
-            parameters.expander->expand(QStringLiteral("%{IDE:ResourcePath}/package-manager")));
+    if (settings(parameters.project).packageManagerAutoSetup()) {
+        const FilePath idePackageManagerDir = Core::ICore::resourcePath("package-manager");
+        const FilePath localPackageManagerDir = buildDirectory / Constants::PACKAGE_MANAGER_DIR;
 
-        if (!localPackageManagerDir.exists() && idePackageManagerDir.exists())
-            idePackageManagerDir.copyRecursively(localPackageManagerDir);
+        if (!idePackageManagerDir.isDir()) {
+            BuildSystem::appendBuildSystemOutput(
+                Tr::tr("Qt Creator installation is missing the "
+                       "package-manager directory. It was expected here: \"%1\".")
+                    .arg(idePackageManagerDir.toUserOutput()));
+        } else if (!localPackageManagerDir.exists()) {
+            const auto result = idePackageManagerDir.copyRecursively(localPackageManagerDir);
+            if (!result) {
+                BuildSystem::appendBuildSystemOutput(
+                    addCMakePrefix(
+                        {Tr::tr("Failed to copy package-manager folder:"), result.error()})
+                        .join('\n'));
+            }
+        }
     }
 
-    const auto parser = new CMakeParser;
+    const auto parser = new CMakeOutputParser;
     parser->setSourceDirectory(parameters.sourceDirectory);
-    m_parser.addLineParser(parser);
+    m_parser.addLineParsers({new CMakeAutogenParser, parser});
+    m_parser.addLineParsers(parameters.outputParsers());
 
     // Always use the sourceDir: If we are triggered because the build directory is getting deleted
     // then we are racing against CMakeCache.txt also getting deleted.
@@ -111,32 +140,46 @@ void CMakeProcess::run(const BuildDirParameters &parameters, const QStringList &
     m_process->setWorkingDirectory(buildDirectory);
     m_process->setEnvironment(parameters.environment);
 
-    m_process->setStdOutLineCallback([](const QString &s) {
-        BuildSystem::appendBuildSystemOutput(stripTrailingNewline(s));
+    m_process->setStdOutLineCallback([this](const QString &s) {
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix(stripTrailingNewline(s)));
+        emit stdOutReady(s);
     });
 
     m_process->setStdErrLineCallback([this](const QString &s) {
         m_parser.appendMessage(s, StdErrFormat);
-        BuildSystem::appendBuildSystemOutput(stripTrailingNewline(s));
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix(stripTrailingNewline(s)));
     });
 
     connect(m_process.get(), &Process::done, this, [this] {
-        handleProcessDone(m_process->resultData());
+        if (m_process->result() != ProcessResult::FinishedWithSuccess) {
+            const QString message = m_process->exitMessage();
+            BuildSystem::appendBuildSystemOutput(addCMakePrefix({{}, message}).join('\n'));
+            TaskHub::addTask(BuildSystemTask(Task::Error, message));
+        }
+
+        emit finished(m_process->exitCode());
+
+        const QString elapsedTime = Utils::formatElapsedTime(m_elapsed.elapsed());
+        BuildSystem::appendBuildSystemOutput(addCMakePrefix({{}, elapsedTime}).join('\n'));
     });
 
     CommandLine commandLine(cmakeExecutable);
-    commandLine.addArgs({"-S", sourceDirectory.path(), "-B", buildDirectory.path()});
+    commandLine.addArgs(
+        {"-S",
+         CMakeToolManager::mappedFilePath(parameters.project, sourceDirectory).path(),
+         "-B",
+         CMakeToolManager::mappedFilePath(parameters.project, buildDirectory).path()});
     commandLine.addArgs(arguments);
 
     TaskHub::clearTasks(ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM);
 
-    BuildSystem::startNewBuildSystemOutput(::CMakeProjectManager::Tr::tr("Running %1 in %2.")
-            .arg(commandLine.toUserOutput(), buildDirectory.toUserOutput()));
+    BuildSystem::startNewBuildSystemOutput(
+        addCMakePrefix(::CMakeProjectManager::Tr::tr("Running %1 in %2.")
+                           .arg(commandLine.toUserOutput(), buildDirectory.toUserOutput())));
 
     ProcessProgress *progress = new ProcessProgress(m_process.get());
     progress->setDisplayName(::CMakeProjectManager::Tr::tr("Configuring \"%1\"")
                              .arg(parameters.projectName));
-    m_process->setTimeoutS(10); // for process progress timeout estimation
     m_process->setCommand(commandLine);
     m_elapsed.start();
     m_process->start();
@@ -148,30 +191,16 @@ void CMakeProcess::stop()
         m_process->stop();
 }
 
-void CMakeProcess::handleProcessDone(const Utils::ProcessResultData &resultData)
+QString addCMakePrefix(const QString &str)
 {
-    const int code = resultData.m_exitCode;
-    QString msg;
-    if (resultData.m_error == QProcess::FailedToStart) {
-        msg = ::CMakeProjectManager::Tr::tr("CMake process failed to start.");
-    } else if (resultData.m_exitStatus != QProcess::NormalExit) {
-        if (resultData.m_canceledByUser)
-            msg = ::CMakeProjectManager::Tr::tr("CMake process was canceled by the user.");
-        else
-            msg = ::CMakeProjectManager::Tr::tr("CMake process crashed.");
-    } else if (code != 0) {
-        msg = ::CMakeProjectManager::Tr::tr("CMake process exited with exit code %1.").arg(code);
-    }
+    static const QString prefix
+        = ansiColoredText(Constants::OUTPUT_PREFIX, creatorColor(Theme::Token_Text_Muted));
+    return prefix + str;
+}
 
-    if (!msg.isEmpty()) {
-        BuildSystem::appendBuildSystemOutput(msg + '\n');
-        TaskHub::addTask(BuildSystemTask(Task::Error, msg));
-    }
-
-    emit finished(code);
-
-    const QString elapsedTime = Utils::formatElapsedTime(m_elapsed.elapsed());
-    BuildSystem::appendBuildSystemOutput(elapsedTime + '\n');
+QStringList addCMakePrefix(const QStringList &list)
+{
+    return Utils::transform(list, [](const QString &str) { return addCMakePrefix(str); });
 }
 
 } // CMakeProjectManager::Internal
