@@ -11,6 +11,8 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 
+using namespace Utils;
+
 namespace CMakeProjectManager::Internal {
 
 bool parseVersion(const QJsonValue &jsonValue, int &version)
@@ -44,7 +46,7 @@ std::optional<QStringList> parseInclude(const QJsonValue &jsonValue)
         if (jsonValue.isArray()) {
             includes = QStringList();
             const QJsonArray includeArray = jsonValue.toArray();
-            for (const QJsonValue &includeValue : includeArray)
+            for (const auto &includeValue : includeArray)
                 includes.value() << includeValue.toString();
         }
     }
@@ -103,7 +105,7 @@ std::optional<PresetsDetails::Condition> parseCondition(const QJsonValue &jsonVa
             if (object.value("list").isArray()) {
                 condition->list = QStringList();
                 const QJsonArray listArray = object.value("list").toArray();
-                for (const QJsonValue &listValue : listArray)
+                for (const auto &listValue : listArray)
                     condition->list.value() << listValue.toString();
             }
         }
@@ -127,7 +129,7 @@ std::optional<PresetsDetails::Condition> parseCondition(const QJsonValue &jsonVa
             if (object.value("conditions").isArray()) {
                 condition->conditions = std::vector<PresetsDetails::Condition::ConditionPtr>();
                 const QJsonArray conditionsArray = object.value("conditions").toArray();
-                for (const QJsonValue &conditionsValue : conditionsArray) {
+                for (const auto &conditionsValue : conditionsArray) {
                     condition->conditions.value().emplace_back(
                         std::make_shared<PresetsDetails::Condition>(
                             parseCondition(conditionsValue).value()));
@@ -148,6 +150,30 @@ std::optional<PresetsDetails::Condition> parseCondition(const QJsonValue &jsonVa
     return condition;
 }
 
+bool parseVendor(const QJsonValue &jsonValue, std::optional<QVariantMap> &vendorSettings)
+{
+    // The whole section is optional
+    if (jsonValue.isUndefined())
+        return true;
+    if (!jsonValue.isObject())
+        return false;
+
+    const QJsonObject object = jsonValue.toObject();
+    const QJsonValue qtIo = object.value("qt.io/QtCreator/1.0");
+    if (qtIo.isUndefined())
+        return true;
+    if (!qtIo.isObject())
+        return false;
+
+    const QJsonObject qtIoObject = qtIo.toObject();
+    vendorSettings = QVariantMap();
+    for (const QString &settingKey : qtIoObject.keys()) {
+        const QJsonValue settingValue = qtIoObject.value(settingKey);
+        vendorSettings->insert(settingKey, settingValue.toVariant());
+    }
+    return true;
+}
+
 bool parseConfigurePresets(const QJsonValue &jsonValue,
                            QList<PresetsDetails::ConfigurePreset> &configurePresets,
                            const Utils::FilePath &fileDir)
@@ -160,7 +186,7 @@ bool parseConfigurePresets(const QJsonValue &jsonValue,
         return false;
 
     const QJsonArray configurePresetsArray = jsonValue.toArray();
-    for (const QJsonValue &presetJson : configurePresetsArray) {
+    for (const auto &presetJson : configurePresetsArray) {
         if (!presetJson.isObject())
             continue;
 
@@ -176,7 +202,7 @@ bool parseConfigurePresets(const QJsonValue &jsonValue,
             preset.inherits = QStringList();
             if (inherits.isArray()) {
                 const QJsonArray inheritsArray = inherits.toArray();
-                for (const QJsonValue &inheritsValue : inheritsArray)
+                for (const auto &inheritsValue : inheritsArray)
                     preset.inherits.value() << inheritsValue.toString();
             } else {
                 QString inheritsValue = inherits.toString();
@@ -188,6 +214,9 @@ bool parseConfigurePresets(const QJsonValue &jsonValue,
         if (object.contains("condition"))
             preset.condition = parseCondition(object.value("condition"));
 
+        if (object.contains("vendor"))
+            parseVendor(object.value("vendor"), preset.vendor);
+
         if (object.contains("displayName"))
             preset.displayName = object.value("displayName").toString();
         if (object.contains("description"))
@@ -201,7 +230,7 @@ bool parseConfigurePresets(const QJsonValue &jsonValue,
         if (object.contains("toolchainFile"))
             preset.toolchainFile = object.value("toolchainFile").toString();
         if (object.contains("cmakeExecutable"))
-            preset.cmakeExecutable = object.value("cmakeExecutable").toString();
+            preset.cmakeExecutable = FilePath::fromUserInput(object.value("cmakeExecutable").toString());
 
         const QJsonObject cacheVariablesObj = object.value("cacheVariables").toObject();
         for (const QString &cacheKey : cacheVariablesObj.keys()) {
@@ -338,9 +367,9 @@ bool parseConfigurePresets(const QJsonValue &jsonValue,
     return true;
 }
 
-bool parseBuildPresets(const QJsonValue &jsonValue,
-                       QList<PresetsDetails::BuildPreset> &buildPresets,
-                       const Utils::FilePath &fileDir)
+static bool parseBuildPresets(const QJsonValue &jsonValue,
+                              QList<PresetsDetails::BuildPreset> &buildPresets,
+                              const FilePath &fileDir)
 {
     // The whole section is optional
     if (jsonValue.isUndefined())
@@ -350,7 +379,7 @@ bool parseBuildPresets(const QJsonValue &jsonValue,
         return false;
 
     const QJsonArray buildPresetsArray = jsonValue.toArray();
-    for (const QJsonValue &presetJson : buildPresetsArray) {
+    for (const auto &presetJson : buildPresetsArray) {
         if (!presetJson.isObject())
             continue;
 
@@ -366,7 +395,7 @@ bool parseBuildPresets(const QJsonValue &jsonValue,
             preset.inherits = QStringList();
             if (inherits.isArray()) {
                 const QJsonArray inheritsArray = inherits.toArray();
-                for (const QJsonValue &inheritsValue : inheritsArray)
+                for (const auto &inheritsValue : inheritsArray)
                     preset.inherits.value() << inheritsValue.toString();
             } else {
                 QString inheritsValue = inherits.toString();
@@ -378,6 +407,9 @@ bool parseBuildPresets(const QJsonValue &jsonValue,
         if (object.contains("condition"))
             preset.condition = parseCondition(object.value("condition"));
 
+        if (object.contains("vendor"))
+            parseVendor(object.value("vendor"), preset.vendor);
+
         if (object.contains("displayName"))
             preset.displayName = object.value("displayName").toString();
         if (object.contains("description"))
@@ -404,7 +436,7 @@ bool parseBuildPresets(const QJsonValue &jsonValue,
             preset.targets = QStringList();
             if (targets.isArray()) {
                 const QJsonArray targetsArray = targets.toArray();
-                for (const QJsonValue &targetsValue : targetsArray)
+                for (const auto &targetsValue : targetsArray)
                     preset.targets.value() << targetsValue.toString();
             } else {
                 QString targetsValue = targets.toString();
@@ -424,7 +456,7 @@ bool parseBuildPresets(const QJsonValue &jsonValue,
             if (nativeToolOptions.isArray()) {
                 preset.nativeToolOptions = QStringList();
                 const QJsonArray toolOptionsArray = nativeToolOptions.toArray();
-                for (const QJsonValue &toolOptionsValue : toolOptionsArray)
+                for (const auto &toolOptionsValue : toolOptionsArray)
                     preset.nativeToolOptions.value() << toolOptionsValue.toString();
             }
         }
@@ -440,11 +472,12 @@ const PresetsData &PresetsParser::presetsData() const
     return m_presetsData;
 }
 
-bool PresetsParser::parse(const Utils::FilePath &jsonFile, QString &errorMessage, int &errorLine)
+bool PresetsParser::parse(const FilePath &jsonFile, QString &errorMessage, int &errorLine)
 {
-    const Utils::expected_str<QByteArray> jsonContents = jsonFile.fileContents();
+    const Utils::Result<QByteArray> jsonContents = jsonFile.fileContents();
     if (!jsonContents) {
-        errorMessage = Tr::tr("Failed to read file \"%1\".").arg(jsonFile.fileName());
+        errorMessage
+            = ::CMakeProjectManager::Tr::tr("Failed to read file \"%1\".").arg(jsonFile.fileName());
         return false;
     }
 
@@ -460,7 +493,8 @@ bool PresetsParser::parse(const Utils::FilePath &jsonFile, QString &errorMessage
     }
 
     if (!jsonDoc.isObject()) {
-        errorMessage = Tr::tr("Invalid file \"%1\".").arg(jsonFile.fileName());
+        errorMessage
+            = ::CMakeProjectManager::Tr::tr("Invalid file \"%1\".").arg(jsonFile.fileName());
         return false;
     }
 
@@ -469,7 +503,8 @@ bool PresetsParser::parse(const Utils::FilePath &jsonFile, QString &errorMessage
     m_presetsData.fileDir = jsonFile.parentDir();
 
     if (!parseVersion(root.value("version"), m_presetsData.version)) {
-        errorMessage = Tr::tr("Invalid \"version\" in file \"%1\".").arg(jsonFile.fileName());
+        errorMessage = ::CMakeProjectManager::Tr::tr("Invalid \"version\" in file \"%1\".")
+                           .arg(jsonFile.fileName());
         return false;
     }
 
@@ -484,8 +519,9 @@ bool PresetsParser::parse(const Utils::FilePath &jsonFile, QString &errorMessage
     if (!parseConfigurePresets(root.value("configurePresets"),
                                m_presetsData.configurePresets,
                                jsonFile.parentDir())) {
-        errorMessage
-            = Tr::tr("Invalid \"configurePresets\" section in %1 file").arg(jsonFile.fileName());
+        errorMessage = ::CMakeProjectManager::Tr::tr(
+                           "Invalid \"configurePresets\" section in file \"%1\".")
+                           .arg(jsonFile.fileName());
         return false;
     }
 
@@ -493,18 +529,24 @@ bool PresetsParser::parse(const Utils::FilePath &jsonFile, QString &errorMessage
     if (!parseBuildPresets(root.value("buildPresets"),
                            m_presetsData.buildPresets,
                            jsonFile.parentDir())) {
-        errorMessage
-            = Tr::tr("Invalid \"buildPresets\" section in %1 file").arg(jsonFile.fileName());
+        errorMessage = ::CMakeProjectManager::Tr::tr(
+                           "Invalid \"buildPresets\" section in file \"%1\".")
+                           .arg(jsonFile.fileName());
         return false;
     }
 
+    // optional
+    if (!parseVendor(root.value("vendor"), m_presetsData.vendor)) {
+        errorMessage = ::CMakeProjectManager::Tr::tr("Invalid \"vendor\" section in file \"%1\".")
+                           .arg(jsonFile.fileName());
+    }
+
     return true;
 }
 
-static QHash<QString, QString> merge(const QHash<QString, QString> &first,
-                                     const QHash<QString, QString> &second)
+static QVariantMap merge(const QVariantMap &first, const QVariantMap &second)
 {
-    QHash<QString, QString> result = first;
+    QVariantMap result = first;
     for (auto it = second.constKeyValueBegin(); it != second.constKeyValueEnd(); ++it) {
         result[it->first] = it->second;
     }
@@ -531,14 +573,14 @@ static QStringList merge(const QStringList &first, const QStringList &second)
 
 void PresetsDetails::ConfigurePreset::inheritFrom(const ConfigurePreset &other)
 {
-    if (!condition && other.condition && !other.condition.value().isNull())
+    if (!condition && other.condition && !other.condition->isNull())
         condition = other.condition;
 
     if (!vendor && other.vendor)
         vendor = other.vendor;
 
     if (vendor && other.vendor)
-        vendor = merge(other.vendor.value(), vendor.value());
+        vendor = merge(*other.vendor, *vendor);
 
     if (!generator && other.generator)
         generator = other.generator;
@@ -564,12 +606,12 @@ void PresetsDetails::ConfigurePreset::inheritFrom(const ConfigurePreset &other)
     if (!cacheVariables && other.cacheVariables)
         cacheVariables = other.cacheVariables;
     else if (cacheVariables && other.cacheVariables)
-        cacheVariables = merge(other.cacheVariables.value(), cacheVariables.value());
+        cacheVariables = merge(*other.cacheVariables, *cacheVariables);
 
     if (!environment && other.environment)
         environment = other.environment;
     else if (environment && other.environment)
-        environment = environment.value().appliedToEnvironment(other.environment.value());
+        environment = environment->appliedToEnvironment(*other.environment);
 
     if (!warnings && other.warnings)
         warnings = other.warnings;
@@ -583,19 +625,19 @@ void PresetsDetails::ConfigurePreset::inheritFrom(const ConfigurePreset &other)
 
 void PresetsDetails::BuildPreset::inheritFrom(const BuildPreset &other)
 {
-    if (!condition && other.condition && !other.condition.value().isNull())
+    if (!condition && other.condition && !other.condition->isNull())
         condition = other.condition;
 
     if (!vendor && other.vendor)
         vendor = other.vendor;
 
     if (vendor && other.vendor)
-        vendor = merge(other.vendor.value(), vendor.value());
+        vendor = merge(*other.vendor, *vendor);
 
     if (!environment && other.environment)
         environment = other.environment;
     else if (environment && other.environment)
-        environment = environment.value().appliedToEnvironment(other.environment.value());
+        environment = environment->appliedToEnvironment(*other.environment);
 
     if (!configurePreset && other.configurePreset)
         configurePreset = other.configurePreset;
@@ -609,7 +651,7 @@ void PresetsDetails::BuildPreset::inheritFrom(const BuildPreset &other)
     if (!targets && other.targets)
         targets = other.targets;
     else if (targets && other.targets)
-        targets = merge(other.targets.value(), targets.value());
+        targets = merge(*other.targets, *targets);
 
     if (!configuration && other.configuration)
         configuration = other.configuration;
@@ -623,7 +665,7 @@ void PresetsDetails::BuildPreset::inheritFrom(const BuildPreset &other)
     if (!nativeToolOptions && other.nativeToolOptions)
         nativeToolOptions = other.nativeToolOptions;
     else if (nativeToolOptions && other.nativeToolOptions)
-        nativeToolOptions = merge(other.nativeToolOptions.value(), nativeToolOptions.value());
+        nativeToolOptions = merge(*other.nativeToolOptions, *nativeToolOptions);
 }
 
 bool PresetsDetails::Condition::evaluate() const
@@ -632,38 +674,38 @@ bool PresetsDetails::Condition::evaluate() const
         return true;
 
     if (isConst() && constValue)
-        return constValue.value();
+        return *constValue;
 
     if (isEquals() && lhs && rhs)
-        return lhs.value() == rhs.value();
+        return *lhs == *rhs;
 
     if (isNotEquals() && lhs && rhs)
-        return lhs.value() != rhs.value();
+        return *lhs != *rhs;
 
     if (isInList() && string && list)
-        return list.value().contains(string.value());
+        return list->contains(*string);
 
     if (isNotInList() && string && list)
-        return !list.value().contains(string.value());
+        return !list->contains(*string);
 
     if (isMatches() && string && regex) {
-        QRegularExpression qRegex(regex.value());
-        return qRegex.match(string.value()).hasMatch();
+        QRegularExpression qRegex(*regex);
+        return qRegex.match(*string).hasMatch();
     }
 
     if (isNotMatches() && string && regex) {
-        QRegularExpression qRegex(regex.value());
-        return !qRegex.match(string.value()).hasMatch();
+        QRegularExpression qRegex(*regex);
+        return !qRegex.match(*string).hasMatch();
     }
 
     if (isAnyOf() && conditions)
-        return Utils::anyOf(conditions.value(), [](const ConditionPtr &c) { return c->evaluate(); });
+        return Utils::anyOf(*conditions, [](const ConditionPtr &c) { return c->evaluate(); });
 
     if (isAllOf() && conditions)
-        return Utils::allOf(conditions.value(), [](const ConditionPtr &c) { return c->evaluate(); });
+        return Utils::allOf(*conditions, [](const ConditionPtr &c) { return c->evaluate(); });
 
     if (isNot() && condition)
-        return !condition.value()->evaluate();
+        return !(*condition)->evaluate();
 
     return false;
 }
