@@ -35,8 +35,11 @@
 
 #include <QMainWindow>
 
+using namespace Tasking;
 using namespace Utils;
 
+using namespace std::chrono;
+
 namespace Core {
 namespace Internal {
 
@@ -45,6 +48,7 @@ static Locator *m_instance = nullptr;
 const char kDirectoryFilterPrefix[] = "directory";
 const char kUrlFilterPrefix[] = "url";
 const char kUseCenteredPopup[] = "UseCenteredPopupForShortcut";
+const char kUseTabCompletion[] = "UseTabCompletion";
 
 class LocatorData
 {
@@ -150,31 +154,47 @@ bool Locator::delayedInitialize()
 void Locator::aboutToShutdown()
 {
     m_refreshTimer.stop();
-    m_taskTree.reset();
+    m_taskTreeRunner.reset();
 }
 
 void Locator::loadSettings()
 {
-    SettingsDatabase *settings = ICore::settingsDatabase();
+    namespace DB = SettingsDatabase;
     // check if we have to read old settings
     // TOOD remove a few versions after 4.15
-    const QString settingsGroup = settings->contains("Locator") ? QString("Locator")
+    const QString settingsGroup = DB::contains("Locator") ? QString("Locator")
                                                                 : QString("QuickOpen");
     const Settings def;
-    settings->beginGroup(settingsGroup);
-    m_refreshTimer.setInterval(settings->value("RefreshInterval", 60).toInt() * 60000);
-    m_settings.useCenteredPopup = settings->value(kUseCenteredPopup, def.useCenteredPopup).toBool();
+    DB::beginGroup(settingsGroup);
+    // TODO SettingsDatabase just for old settings from QtC < 16.0
+    // when removed, the defaults need to move to the ICore::settings() code path below
+    {
+        m_refreshTimer.setInterval(
+            minutes(DB::value("RefreshInterval", int(def.refreshInterval / 1min)).toInt()));
+        m_settings.relativePaths = DB::value("RelativePaths", def.relativePaths).toBool();
+        m_settings.useCenteredPopup = DB::value(kUseCenteredPopup, def.useCenteredPopup).toBool();
+    }
+    ICore::settings()->withGroup("Locator", [this](QtcSettings *settings) {
+        m_refreshTimer.setInterval(
+            minutes(settings->value("RefreshInterval", refreshInterval()).toInt()));
+        m_settings.relativePaths
+            = settings->value("RelativePaths", m_settings.relativePaths).toBool();
+        m_settings.useCenteredPopup
+            = settings->value(kUseCenteredPopup, m_settings.useCenteredPopup).toBool();
+        m_settings.useTabCompletion
+            = settings->value(kUseTabCompletion, m_settings.useTabCompletion).toBool();
+    });
 
     for (ILocatorFilter *filter : std::as_const(m_filters)) {
-        if (settings->contains(filter->id().toString())) {
-            const QByteArray state = settings->value(filter->id().toString()).toByteArray();
+        if (DB::contains(filter->id().toString())) {
+            const QByteArray state = DB::value(filter->id().toString()).toByteArray();
             if (!state.isEmpty())
                 filter->restoreState(state);
         }
     }
-    settings->beginGroup("CustomFilters");
+    DB::beginGroup("CustomFilters");
     QList<ILocatorFilter *> customFilters;
-    const QStringList keys = settings->childKeys();
+    const QStringList keys = DB::childKeys();
     int count = 0;
     const Id directoryBaseId(Constants::CUSTOM_DIRECTORY_FILTER_BASEID);
     const Id urlBaseId(Constants::CUSTOM_URL_FILTER_BASEID);
@@ -188,12 +208,12 @@ void Locator::loadSettings()
             urlFilter->setIsCustomFilter(true);
             filter = urlFilter;
         }
-        filter->restoreState(settings->value(key).toByteArray());
+        filter->restoreState(DB::value(key).toByteArray());
         customFilters.append(filter);
     }
     setCustomFilters(customFilters);
-    settings->endGroup();
-    settings->endGroup();
+    DB::endGroup();
+    DB::endGroup();
 
     if (m_refreshTimer.interval() > 0)
         m_refreshTimer.start();
@@ -210,17 +230,15 @@ void Locator::updateFilterActions()
         if (filter->shortcutString().isEmpty() || filter->isHidden())
             continue;
         Id filterId = filter->id();
-        Id actionId = filter->actionId();
         QAction *action = nullptr;
         if (!actionCopy.contains(filterId)) {
             // register new action
-            action = new QAction(filter->displayName(), this);
-            Command *cmd = ActionManager::registerAction(action, actionId);
-            cmd->setAttribute(Command::CA_UpdateText);
-            cmd->setDefaultKeySequence(filter->defaultKeySequence());
-            connect(action, &QAction::triggered, this, [filter] {
-                LocatorManager::showFilter(filter);
-            });
+            ActionBuilder(this, filter->actionId())
+                .setText(filter->displayName())
+                .bindContextAction(&action)
+                .setCommandAttribute(Command::CA_UpdateText)
+                .setDefaultKeySequence(filter->defaultKeySequence())
+                .addOnTriggered(this, [filter] { LocatorManager::showFilter(filter); });
         } else {
             action = actionCopy.take(filterId);
             action->setText(filter->displayName());
@@ -288,19 +306,23 @@ void Locator::saveSettings() const
         return;
 
     const Settings def;
-    SettingsDatabase *s = ICore::settingsDatabase();
-    s->beginTransaction();
-    s->beginGroup("Locator");
-    s->remove(QString());
-    s->setValue("RefreshInterval", refreshInterval());
-    s->setValueWithDefault(kUseCenteredPopup, m_settings.useCenteredPopup, def.useCenteredPopup);
+    namespace DB = SettingsDatabase;
+    DB::beginTransaction();
+    DB::beginGroup("Locator");
+    DB::remove(QString());
+    // TODO SettingsDatabase only for backward compatibility < 16.0
+    {
+        DB::setValue("RefreshInterval", refreshInterval());
+        DB::setValue("RelativePaths", relativePaths());
+        DB::setValueWithDefault(kUseCenteredPopup, m_settings.useCenteredPopup, def.useCenteredPopup);
+    }
     for (ILocatorFilter *filter : m_filters) {
         if (!m_customFilters.contains(filter) && filter->id().isValid()) {
             const QByteArray state = filter->saveState();
-            s->setValueWithDefault(filter->id().toString(), state);
+            DB::setValueWithDefault(filter->id().toString(), state);
         }
     }
-    s->beginGroup("CustomFilters");
+    DB::beginGroup("CustomFilters");
     int i = 0;
     for (ILocatorFilter *filter : m_customFilters) {
         const char *prefix = filter->id().name().startsWith(
@@ -308,12 +330,22 @@ void Locator::saveSettings() const
                                  ? kDirectoryFilterPrefix
                                  : kUrlFilterPrefix;
         const QByteArray state = filter->saveState();
-        s->setValueWithDefault(prefix + QString::number(i), state);
+        DB::setValueWithDefault(prefix + QString::number(i), state);
         ++i;
     }
-    s->endGroup();
-    s->endGroup();
-    s->endTransaction();
+    DB::endGroup();
+    DB::endGroup();
+    DB::endTransaction();
+
+    ICore::settings()->withGroup("Locator", [this, &def](QtcSettings *settings) {
+        settings->setValueWithDefault(
+            "RefreshInterval", refreshInterval(), int(def.refreshInterval / 1min));
+        settings->setValueWithDefault("RelativePaths", m_settings.relativePaths, def.relativePaths);
+        settings->setValueWithDefault(
+            kUseCenteredPopup, m_settings.useCenteredPopup, def.useCenteredPopup);
+        settings->setValueWithDefault(
+            kUseTabCompletion, m_settings.useTabCompletion, def.useTabCompletion);
+    });
 }
 
 /*!
@@ -358,10 +390,20 @@ void Locator::setRefreshInterval(int interval)
         m_refreshTimer.setInterval(0);
         return;
     }
-    m_refreshTimer.setInterval(interval * 60000);
+    m_refreshTimer.setInterval(minutes(interval));
     m_refreshTimer.start();
 }
 
+bool Locator::relativePaths() const
+{
+    return m_settings.relativePaths;
+}
+
+void Locator::setRelativePaths(bool use)
+{
+    m_settings.relativePaths = use;
+}
+
 bool Locator::useCenteredPopupForShortcut()
 {
     return m_instance->m_settings.useCenteredPopup;
@@ -372,40 +414,47 @@ void Locator::setUseCenteredPopupForShortcut(bool center)
     m_instance->m_settings.useCenteredPopup = center;
 }
 
+bool Locator::useTabCompletion()
+{
+    return m_instance->m_settings.useTabCompletion;
+}
+
+void Locator::setUseTabCompletion(bool useTabCompletion)
+{
+    m_instance->m_settings.useTabCompletion = useTabCompletion;
+}
+
 void Locator::refresh(const QList<ILocatorFilter *> &filters)
 {
     if (ExtensionSystem::PluginManager::isShuttingDown())
         return;
 
-    m_taskTree.reset(); // Superfluous, just for clarity. The next reset() below is enough.
+    m_taskTreeRunner.reset(); // Superfluous, just for clarity. The start() below is enough.
     m_refreshingFilters = Utils::filteredUnique(m_refreshingFilters + filters);
 
-    using namespace Tasking;
-    QList<GroupItem> tasks{parallel};
+    const auto onTreeSetup = [](TaskTree *taskTree) {
+        auto progress = new TaskProgress(taskTree);
+        progress->setDisplayName(Tr::tr("Updating Locator Caches"));
+    };
+    const auto onTreeDone = [this](DoneWith result) {
+        if (result == DoneWith::Success)
+            saveSettings();
+    };
+
+    GroupItems tasks{parallel};
     for (ILocatorFilter *filter : std::as_const(m_refreshingFilters)) {
         const auto task = filter->refreshRecipe();
         if (!task.has_value())
             continue;
 
         const Group group {
-            finishAllAndDone,
+            finishAllAndSuccess,
             *task,
-            onGroupDone([this, filter] { m_refreshingFilters.removeOne(filter); })
+            onGroupDone([this, filter] { m_refreshingFilters.removeOne(filter); }, CallDoneIf::Success)
         };
         tasks.append(group);
     }
-
-    m_taskTree.reset(new TaskTree{tasks});
-    connect(m_taskTree.get(), &TaskTree::done, this, [this] {
-        saveSettings();
-        m_taskTree.release()->deleteLater();
-    });
-    connect(m_taskTree.get(), &TaskTree::errorOccurred, this, [this] {
-        m_taskTree.release()->deleteLater();
-    });
-    auto progress = new TaskProgress(m_taskTree.get());
-    progress->setDisplayName(Tr::tr("Updating Locator Caches"));
-    m_taskTree->start();
+    m_taskTreeRunner.start(tasks, onTreeSetup, onTreeDone);
 }
 
 void Locator::showFilter(ILocatorFilter *filter, LocatorWidget *widget)
