@@ -14,7 +14,7 @@
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/buildstep.h>
 #include <projectexplorer/buildsteplist.h>
-#include <projectexplorer/kitaspects.h>
+#include <projectexplorer/environmentkitaspect.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/runconfiguration.h>
@@ -44,8 +44,8 @@ static bool isDisabled(TriStateAspect *aspect)
     return aspect->value() == TriState::Disabled;
 }
 
-DebuggerRunConfigurationAspect::DebuggerRunConfigurationAspect(Target *target)
-    : m_target(target)
+DebuggerRunConfigurationAspect::DebuggerRunConfigurationAspect(BuildConfiguration *bc)
+    : m_buildConfiguration(bc)
 {
     setId("DebuggerAspect");
     setDisplayName(Tr::tr("Debugger settings"));
@@ -72,7 +72,7 @@ DebuggerRunConfigurationAspect::DebuggerRunConfigurationAspect(Target *target)
         details->setState(DetailsWidget::Expanded);
         auto innerPane = new QWidget;
         details->setWidget(innerPane);
-        builder.addItem(Layouting::noMargin);
+        builder.setNoMargins();
         builder.attachTo(innerPane);
 
         const auto setSummaryText = [this, details] {
@@ -100,10 +100,10 @@ DebuggerRunConfigurationAspect::DebuggerRunConfigurationAspect(Target *target)
         };
         setSummaryText();
 
-        connect(&m_cppAspect, &BaseAspect::changed, this, setSummaryText);
-        connect(&m_qmlAspect, &BaseAspect::changed, this, setSummaryText);
-        connect(&m_pythonAspect, &BaseAspect::changed, this, setSummaryText);
-        connect(&m_overrideStartupAspect, &BaseAspect::changed, this, setSummaryText);
+        connect(&m_cppAspect, &BaseAspect::changed, details, setSummaryText);
+        connect(&m_qmlAspect, &BaseAspect::changed, details, setSummaryText);
+        connect(&m_pythonAspect, &BaseAspect::changed, details, setSummaryText);
+        connect(&m_overrideStartupAspect, &BaseAspect::changed, details, setSummaryText);
 
         return details;
     });
@@ -116,15 +116,15 @@ DebuggerRunConfigurationAspect::DebuggerRunConfigurationAspect(Target *target)
 
     m_cppAspect.setSettingsKey("RunConfiguration.UseCppDebugger");
     m_cppAspect.setLabelText(Tr::tr("C++ debugger:"));
-    m_cppAspect.setOptionTexts(Tr::tr("Enabled"), Tr::tr("Disabled"), Tr::tr("Automatic"));
+    m_cppAspect.setOptionText(TriState::DefaultValue, Tr::tr("Automatic"));
 
     m_qmlAspect.setSettingsKey("RunConfiguration.UseQmlDebugger");
     m_qmlAspect.setLabelText(Tr::tr("QML debugger:"));
-    m_qmlAspect.setOptionTexts(Tr::tr("Enabled"), Tr::tr("Disabled"), Tr::tr("Automatic"));
+    m_qmlAspect.setOptionText(TriState::DefaultValue, Tr::tr("Automatic"));
 
     m_pythonAspect.setSettingsKey("RunConfiguration.UsePythonDebugger");
     m_pythonAspect.setLabelText(Tr::tr("Python debugger:"));
-    m_pythonAspect.setOptionTexts(Tr::tr("Enabled"), Tr::tr("Disabled"), Tr::tr("Automatic"));
+    m_pythonAspect.setOptionText(TriState::DefaultValue, Tr::tr("Automatic"));
 
     // Make sure at least one of the debuggers is set to be active.
     connect(&m_cppAspect, &TriStateAspect::changed, this, [this] {
@@ -159,7 +159,7 @@ void DebuggerRunConfigurationAspect::setUseQmlDebugger(bool value)
 bool DebuggerRunConfigurationAspect::useCppDebugger() const
 {
     if (m_cppAspect() == TriState::Default)
-        return m_target->project()->projectLanguages().contains(
+        return m_buildConfiguration->project()->projectLanguages().contains(
                     ProjectExplorer::Constants::CXX_LANGUAGE_ID);
     return m_cppAspect() == TriState::Enabled;
 }
@@ -182,15 +182,13 @@ static bool projectHasQmlDefines(ProjectExplorer::Project *project)
 bool DebuggerRunConfigurationAspect::useQmlDebugger() const
 {
     if (m_qmlAspect() == TriState::Default) {
-        const Core::Context languages = m_target->project()->projectLanguages();
+        const Core::Context languages = m_buildConfiguration->project()->projectLanguages();
         if (!languages.contains(ProjectExplorer::Constants::QMLJS_LANGUAGE_ID))
-            return projectHasQmlDefines(m_target->project());
+            return projectHasQmlDefines(m_buildConfiguration->project());
 
         // Try to find a build configuration to check whether qml debugging is enabled there
-        if (BuildConfiguration *bc = m_target->activeBuildConfiguration()) {
-            if (const auto aspect = bc->aspect<QtSupport::QmlDebuggingAspect>())
-                return aspect->value() == TriState::Enabled;
-        }
+        if (const auto aspect = m_buildConfiguration->aspect<QtSupport::QmlDebuggingAspect>())
+            return aspect->value() == TriState::Enabled;
 
         return !languages.contains(ProjectExplorer::Constants::CXX_LANGUAGE_ID);
     }
@@ -200,7 +198,7 @@ bool DebuggerRunConfigurationAspect::useQmlDebugger() const
 bool DebuggerRunConfigurationAspect::usePythonDebugger() const
 {
     if (m_pythonAspect() == TriState::Default) {
-        const Core::Context languages = m_target->project()->projectLanguages();
+        const Core::Context languages = m_buildConfiguration->project()->projectLanguages();
         return languages.contains(ProjectExplorer::Constants::PYTHON_LANGUAGE_ID);
     }
     return m_pythonAspect() == TriState::Enabled;
@@ -221,16 +219,6 @@ QString DebuggerRunConfigurationAspect::overrideStartup() const
     return m_overrideStartupAspect();
 }
 
-int DebuggerRunConfigurationAspect::portsUsedByDebugger() const
-{
-    int ports = 0;
-    if (useQmlDebugger())
-        ++ports;
-    if (useCppDebugger())
-        ++ports;
-    return ports;
-}
-
 void DebuggerRunConfigurationAspect::toMap(Store &map) const
 {
     m_cppAspect.toMap(map);
