@@ -1,41 +1,20 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "task.h"
 
 #include "fileinsessionfinder.h"
 #include "projectexplorerconstants.h"
+#include "projectexplorertr.h"
 
-#include <app/app_version.h>
+#include <texteditor/fontsettings.h>
 #include <texteditor/textmark.h>
-
 #include <utils/algorithm.h>
 #include <utils/utilsicons.h>
 #include <utils/qtcassert.h>
 
 #include <QFileInfo>
+#include <QGuiApplication>
 #include <QTextStream>
 
 using namespace Utils;
@@ -81,16 +60,16 @@ Task::Task(TaskType type_, const QString &description,
 Task Task::compilerMissingTask()
 {
     return BuildSystemTask(Task::Error,
-                           tr("%1 needs a compiler set up to build. "
-                              "Configure a compiler in the kit options.")
-                           .arg(Core::Constants::IDE_DISPLAY_NAME));
+                           Tr::tr("%1 needs a compiler set up to build. "
+                                  "Configure a compiler in the kit options.")
+                               .arg(QGuiApplication::applicationDisplayName()));
 }
 
 void Task::setMark(TextEditor::TextMark *mark)
 {
     QTC_ASSERT(mark, return);
-    QTC_ASSERT(m_mark.isNull(), return);
-    m_mark = QSharedPointer<TextEditor::TextMark>(mark);
+    QTC_ASSERT(!m_mark, return);
+    m_mark = std::shared_ptr<TextEditor::TextMark>(mark);
 }
 
 bool Task::isNull() const
@@ -107,10 +86,11 @@ void Task::clear()
     file = Utils::FilePath();
     line = -1;
     movedLine = -1;
+    column = 0;
     category = Utils::Id();
     m_icon = QIcon();
     formats.clear();
-    m_mark.clear();
+    m_mark.reset();
 }
 
 void Task::setFile(const Utils::FilePath &file_)
@@ -125,11 +105,16 @@ void Task::setFile(const Utils::FilePath &file_)
     }
 }
 
-QString Task::description() const
+QString Task::description(DescriptionTags tags) const
 {
-    QString desc = summary;
-    if (!details.isEmpty())
-        desc.append('\n').append(details.join('\n'));
+    QString desc;
+    if (tags & WithSummary)
+        desc = summary;
+    if (!details.isEmpty()) {
+        if (!desc.isEmpty())
+            desc.append('\n');
+        desc.append(details.join('\n'));
+    }
     return desc;
 }
 
@@ -140,6 +125,54 @@ QIcon Task::icon() const
     return m_icon;
 }
 
+void Task::setIcon(const QIcon &icon)
+{
+    m_icon = icon;
+}
+
+QString Task::formattedDescription(DescriptionTags tags, const QString &extraHeading) const
+{
+    if (isNull())
+        return {};
+
+    QString text = description(tags);
+    const int offset = (tags & WithSummary) ? 0 : summary.size() + 1;
+    static const QString linkTagStartPlaceholder("__QTC_LINK_TAG_START__");
+    static const QString linkTagEndPlaceholder("__QTC_LINK_TAG_END__");
+    static const QString linkEndPlaceholder("__QTC_LINK_END__");
+    if (tags & WithLinks) {
+        for (auto formatRange = formats.crbegin(); formatRange != formats.crend(); ++formatRange) {
+            if (!formatRange->format.isAnchor())
+                continue;
+            text.insert(formatRange->start - offset + formatRange->length, linkEndPlaceholder);
+            text.insert(formatRange->start - offset, QString::fromLatin1("%1%2%3").arg(
+                linkTagStartPlaceholder, formatRange->format.anchorHref(), linkTagEndPlaceholder));
+        }
+    }
+    text = text.toHtmlEscaped();
+    if (tags & WithLinks) {
+        text.replace(linkEndPlaceholder, "</a>");
+        text.replace(linkTagStartPlaceholder, "<a href=\"");
+        text.replace(linkTagEndPlaceholder, "\">");
+    }
+    const QString htmlExtraHeading = extraHeading.isEmpty()
+                                         ? QString()
+                                         : QString::fromUtf8("<b>%1</b><br/>").arg(extraHeading);
+    return QString::fromUtf8("<html><body>%1<code style=\"white-space:pre;font-family:%2\">"
+                             "%3</code></body></html>")
+        .arg(htmlExtraHeading, TextEditor::FontSettings::defaultFixedFontFamily(), text);
+}
+
+void Task::addLinkDetail(const QString &link)
+{
+    details.append(link);
+    QTextCharFormat format;
+    format.setAnchor(true);
+    format.setAnchorHref(link);
+    const int offset = summary.length() + 1;
+    formats << QTextLayout::FormatRange{offset, int(link.length()), format};
+}
+
 //
 // functions
 //
@@ -171,7 +204,7 @@ bool operator<(const Task &a, const Task &b)
 }
 
 
-uint qHash(const Task &task)
+size_t qHash(const Task &task)
 {
     return task.taskId;
 }
@@ -185,10 +218,10 @@ QString toHtml(const Tasks &issues)
         str << "<b>";
         switch (t.type) {
         case Task::Error:
-            str << QCoreApplication::translate("ProjectExplorer::Kit", "Error:") << " ";
+            str << Tr::tr("Error:") << " ";
             break;
         case Task::Warning:
-            str << QCoreApplication::translate("ProjectExplorer::Kit", "Warning:") << " ";
+            str << Tr::tr("Warning:") << " ";
             break;
         case Task::Unknown:
         default:
