@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmakenodes.h"
 
@@ -33,12 +11,12 @@
 #include <projectexplorer/target.h>
 
 #include <qtsupport/baseqtversion.h>
-#include <qtsupport/qtkitinformation.h>
+#include <qtsupport/qtkitaspect.h>
 
 #include <resourceeditor/resourcenode.h>
 
+#include <utils/fileutils.h>
 #include <utils/qtcassert.h>
-#include <utils/stringutils.h>
 
 #include <android/androidconstants.h>
 #include <ios/iosconstants.h>
@@ -136,15 +114,14 @@ bool QmakeBuildSystem::supportsAction(Node *context, ProjectAction action, const
                 // A virtual folder, we do what the projectexplorer does
                 const FolderNode *folder = node->asFolderNode();
                 if (folder) {
-                    QStringList list;
-                    foreach (FolderNode *f, folder->folderNodes())
-                        list << f->filePath().toString() + QLatin1Char('/');
-                    if (n->deploysFolder(Utils::commonPath(list)))
+                    FilePaths list;
+                    folder->forEachFolderNode([&](FolderNode *f) { list << f->filePath(); });
+                    if (n->deploysFolder(FileUtils::commonPath(list).toUrlishString()))
                         addExistingFiles = false;
                 }
             }
 
-            addExistingFiles = addExistingFiles && !n->deploysFolder(node->filePath().toString());
+            addExistingFiles = addExistingFiles && !n->deploysFolder(node->filePath().toUrlishString());
 
             if (action == AddExistingFile || action == AddExistingDirectory)
                 return addExistingFiles;
@@ -190,7 +167,7 @@ bool QmakePriFileNode::removeSubProject(const FilePath &proFilePath)
 
 QStringList QmakePriFileNode::subProjectFileNamePatterns() const
 {
-    return QStringList("*.pro");
+    return {"*.pro"};
 }
 
 bool QmakeBuildSystem::addFiles(Node *context, const FilePaths &filePaths, FilePaths *notAdded)
@@ -214,7 +191,7 @@ bool QmakeBuildSystem::addFiles(Node *context, const FilePaths &filePaths, FileP
         FilePath::removeDuplicates(alreadyPresentFiles);
 
         FilePaths actualFilePaths = filePaths;
-        for (const FilePath &e : alreadyPresentFiles)
+        for (const FilePath &e : std::as_const(alreadyPresentFiles))
             actualFilePaths.removeOne(e);
         if (notAdded)
             *notAdded = alreadyPresentFiles;
@@ -237,7 +214,7 @@ RemovedFilesFromProject QmakeBuildSystem::removeFiles(Node *context, const FileP
         FilePaths wildcardFiles;
         FilePaths nonWildcardFiles;
         for (const FilePath &file : filePaths) {
-            if (pri->proFile()->isFileFromWildcard(file.toString()))
+            if (pri->proFile()->isFileFromWildcard(file.toUrlishString()))
                 wildcardFiles << file;
             else
                 nonWildcardFiles << file;
@@ -277,16 +254,27 @@ bool QmakeBuildSystem::canRenameFile(Node *context,
     return BuildSystem::canRenameFile(context, oldFilePath, newFilePath);
 }
 
-bool QmakeBuildSystem::renameFile(Node *context,
-                                  const FilePath &oldFilePath,
-                                  const FilePath &newFilePath)
+bool QmakeBuildSystem::renameFiles(Node *context, const FilePairs &filesToRename, FilePaths *notRenamed)
 {
     if (auto n = dynamic_cast<QmakePriFileNode *>(context)) {
         QmakePriFile *pri = n->priFile();
-        return pri ? pri->renameFile(oldFilePath, newFilePath) : false;
+        if (!pri) {
+            if (notRenamed)
+                *notRenamed = firstPaths(filesToRename);
+            return false;
+        }
+        bool success = true;
+        for (const auto &[oldFilePath, newFilePath] : filesToRename) {
+            if (!pri->renameFile(oldFilePath, newFilePath)) {
+                success = false;
+                if (notRenamed)
+                    *notRenamed << oldFilePath;
+            }
+        }
+        return success;
     }
 
-    return BuildSystem::renameFile(context, oldFilePath, newFilePath);
+    return BuildSystem::renameFiles(context, filesToRename, notRenamed);
 }
 
 bool QmakeBuildSystem::addDependencies(Node *context, const QStringList &dependencies)
@@ -330,7 +318,7 @@ bool QmakeProFileNode::showInSimpleTree() const
 
 QString QmakeProFileNode::buildKey() const
 {
-    return filePath().toString();
+    return filePath().toUrlishString();
 }
 
 bool QmakeProFileNode::parseInProgress() const
@@ -363,7 +351,7 @@ QStringList QmakeProFileNode::targetApplications() const
     return apps;
 }
 
-QVariant QmakeProFileNode::data(Utils::Id role) const
+QVariant QmakeProFileNode::data(Id role) const
 {
     if (role == Android::Constants::AndroidAbis)
         return variableValue(Variable::AndroidAbis);
@@ -377,11 +365,11 @@ QVariant QmakeProFileNode::data(Utils::Id role) const
         return singleVariableValue(Variable::AndroidDeploySettingsFile);
     if (role == Android::Constants::AndroidSoLibPath) {
         TargetInformation info = targetInformation();
-        QStringList res = {info.buildDir.toString()};
+        QStringList res = {info.buildDir.toUrlishString()};
         FilePath destDir = info.destDir;
         if (!destDir.isEmpty()) {
             destDir = info.buildDir.resolvePath(destDir.path());
-            res.append(destDir.toString());
+            res.append(destDir.toUrlishString());
         }
         res.removeDuplicates();
         return res;
@@ -392,12 +380,6 @@ QVariant QmakeProFileNode::data(Utils::Id role) const
     if (role == Android::Constants::AndroidApk)
         return {};
 
-    // We can not use AppMan headers even at build time.
-    if (role == "AppmanPackageDir")
-        return singleVariableValue(Variable::AppmanPackageDir);
-    if (role == "AppmanManifest")
-        return singleVariableValue(Variable::AppmanManifest);
-
     if (role == Ios::Constants::IosTarget) {
         const TargetInformation info = targetInformation();
         if (info.valid)
@@ -407,7 +389,7 @@ QVariant QmakeProFileNode::data(Utils::Id role) const
     if (role == Ios::Constants::IosBuildDir) {
         const TargetInformation info = targetInformation();
         if (info.valid)
-            return info.buildDir.toString();
+            return info.buildDir.toUrlishString();
     }
 
     if (role == Ios::Constants::IosCmakeGenerator) {
@@ -422,21 +404,20 @@ QVariant QmakeProFileNode::data(Utils::Id role) const
     return {};
 }
 
-bool QmakeProFileNode::setData(Utils::Id role, const QVariant &value) const
+bool QmakeProFileNode::setData(Id role, const QVariant &value) const
 {
     QmakeProFile *pro = proFile();
     if (!pro)
         return false;
     QString scope;
     int flags = QmakeProjectManager::Internal::ProWriter::ReplaceValues;
-    if (Target *target = m_buildSystem->target()) {
-        QtSupport::QtVersion *version = QtSupport::QtKitAspect::qtVersion(target->kit());
-        if (version && !version->supportsMultipleQtAbis()) {
-            const QString arch = pro->singleVariableValue(Variable::AndroidAbi);
-            scope = QString("contains(%1,%2)").arg(Android::Constants::ANDROID_TARGET_ARCH)
-                                              .arg(arch);
-            flags |= QmakeProjectManager::Internal::ProWriter::MultiLine;
-        }
+    const QtSupport::QtVersion * const version = QtSupport::QtKitAspect::qtVersion(
+        m_buildSystem->kit());
+    if (version && !version->supportsMultipleQtAbis()) {
+        const QString arch = pro->singleVariableValue(Variable::AndroidAbi);
+        scope = QString("contains(%1,%2)").arg(Android::Constants::ANDROID_TARGET_ARCH)
+                    .arg(arch);
+        flags |= QmakeProjectManager::Internal::ProWriter::MultiLine;
     }
 
     if (role == Android::Constants::AndroidExtraLibs)
