@@ -1,41 +1,21 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmljshoverhandler.h"
+
 #include "qmljseditor.h"
-#include "qmljseditorconstants.h"
 #include "qmljseditordocument.h"
-#include "qmlexpressionundercursor.h"
+#include "qmljseditortr.h"
+#include "qmllsclientsettings.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/helpitem.h>
 #include <coreplugin/helpmanager.h>
-#include <utils/qtcassert.h>
+
 #include <extensionsystem/pluginmanager.h>
+
 #include <qmljs/qmljscontext.h>
 #include <qmljs/qmljsscopechain.h>
 #include <qmljs/qmljsinterpreter.h>
@@ -43,29 +23,31 @@
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsastfwd_p.h>
 #include <qmljs/qmljsutils.h>
+
 #include <texteditor/texteditor.h>
-#include <utils/executeondestruction.h>
+
+#include <utils/qtcassert.h>
 #include <utils/qrcparser.h>
 #include <utils/tooltip/tooltip.h>
 
 #include <QDir>
 #include <QList>
-#include <QStringRef>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
+#include <QScopeGuard>
 
 using namespace Core;
 using namespace QmlJS;
+using namespace QmlJSEditor::Internal;
 using namespace TextEditor;
 
 namespace QmlJSEditor {
-namespace Internal {
 
 namespace {
 
     QString textAt(const Document::Ptr doc,
-                   const AST::SourceLocation &from,
-                   const AST::SourceLocation &to)
+                   const SourceLocation &from,
+                   const SourceLocation &to)
     {
         return doc->source().mid(from.offset, to.end() - from.begin());
     }
@@ -126,11 +108,11 @@ static inline QString getModuleName(const ScopeChain &scopeChain, const Document
             return moduleName + QString::number(majorVersion) + QLatin1Char('.')
                     + QString::number(minorVersion) ;
         } else if (importInfo.isValid() && importInfo.type() == ImportType::Directory) {
-            const QString path = importInfo.path();
-            const QDir dir(qmlDocument->path());
+            const Utils::FilePath path = Utils::FilePath::fromString(importInfo.path());
+            const Utils::FilePath dir = qmlDocument->path();
             // should probably try to make it relatve to some import path, not to the document path
-            QString relativeDir = dir.relativeFilePath(path);
-            const QString name = relativeDir.replace(QLatin1Char('/'), QLatin1Char('.'));
+            const Utils::FilePath relativePath = path.relativeChildPath(dir);
+            const QString name = relativePath.path().replace(QLatin1Char('/'), QLatin1Char('.'));
             return name;
         } else if (importInfo.isValid() && importInfo.type() == ImportType::QrcDirectory) {
             QString path = Utils::QrcParser::normalizedQrcDirectoryPath(importInfo.path());
@@ -145,12 +127,15 @@ static inline QString getModuleName(const ScopeChain &scopeChain, const Document
 bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Document::Ptr &qmlDocument,
                                        const ObjectValue *value, const QStringList &qName)
 {
-    QString moduleName = getModuleName(scopeChain, qmlDocument, value);
+    const QString moduleName = getModuleName(scopeChain, qmlDocument, value);
+    static const QRegularExpression anyVersion("((-1|\\d+)\\.-1)|(\\d+\\.\\d+)$");
 
     QStringList helpIdCandidates;
 
     QStringList helpIdPieces(qName);
-    helpIdPieces.prepend(moduleName);
+    QString strippedModuleName = moduleName;
+    strippedModuleName.remove(anyVersion);
+    helpIdPieces.prepend(strippedModuleName);
     helpIdPieces.prepend("QML");
     helpIdCandidates += helpIdPieces.join('.');
 
@@ -166,15 +151,18 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
     helpIdPieces.removeAt(1);
     helpIdCandidates += helpIdPieces.join('.');
 
-    const HelpItem helpItem(helpIdCandidates, qName.join('.'), HelpItem::QmlComponent);
+    const HelpItem helpItem(helpIdCandidates,
+                            qmlDocument->fileName(),
+                            qName.join('.'),
+                            HelpItem::QmlComponent);
     const HelpItem::Links links = helpItem.links();
 
     // Check if the module name contains a major version.
-    QRegularExpression version("^([^\\d]*)(\\d+)\\.*\\d*$");
-    QRegularExpressionMatch m = version.match(moduleName);
+    static const QRegularExpression version("^([^\\d]*)(\\d+)\\.*\\d*$");
+    const QRegularExpressionMatch m = version.match(moduleName);
     if (m.hasMatch()) {
         QMap<QString, QUrl> filteredUrlMap;
-        QStringRef maj = m.capturedRef(2);
+        const QString maj = m.captured(2);
         for (const HelpItem::Link &link : links) {
             QString urlModuleName = link.second.path().split('/')[1];
             if (urlModuleName.contains(maj))
@@ -195,7 +183,7 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
 
 void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos, ReportPriority report)
 {
-    Utils::ExecuteOnDestruction reportPriority([this, report](){ report(priority()); });
+    const QScopeGuard cleanup([this, report] { report(priority()); });
 
     reset();
 
@@ -270,15 +258,17 @@ void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos, R
 
 bool QmlJSHoverHandler::matchDiagnosticMessage(QmlJSEditorWidget *qmlEditor, int pos)
 {
-    foreach (const QTextEdit::ExtraSelection &sel,
-             qmlEditor->extraSelections(TextEditorWidget::CodeWarningsSelection)) {
+    const QList<QTextEdit::ExtraSelection> selections =
+        qmlEditor->extraSelections(TextEditorWidget::CodeWarningsSelection);
+    for (const QTextEdit::ExtraSelection &sel : selections) {
         if (pos >= sel.cursor.selectionStart() && pos <= sel.cursor.selectionEnd()) {
             setToolTip(sel.format.toolTip());
             return true;
         }
     }
-    foreach (const QTextLayout::FormatRange &range,
-             qmlEditor->qmlJsEditorDocument()->diagnosticRanges()) {
+    const QVector<QTextLayout::FormatRange> ranges =
+        qmlEditor->qmlJsEditorDocument()->diagnosticRanges();
+    for (const QTextLayout::FormatRange &range : ranges) {
         if (pos >= range.start && pos < range.start+range.length) {
             setToolTip(range.format.toolTip());
             return true;
@@ -311,7 +301,9 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
     if (auto binding = AST::cast<const AST::UiScriptBinding *>(member)) {
         if (binding->qualifiedId && posIsInSource(pos, binding->statement)) {
             value = scopeChain.evaluate(binding->qualifiedId);
-            if (value && value->asColorValue()) {
+            if (value && (value->asColorValue()
+                          || (value->asCppComponentValue()
+                              && value->asCppComponentValue()->className() == "color"))) {
                 color = textAt(qmlDocument,
                                binding->statement->firstSourceLocation(),
                                binding->statement->lastSourceLocation());
@@ -359,18 +351,19 @@ void QmlJSHoverHandler::handleImport(const ScopeChain &scopeChain, AST::UiImport
     if (!imports)
         return;
 
-    foreach (const Import &import, imports->all()) {
+    const QList<Import> importList = imports->all();
+    for (const Import &import : importList) {
         if (import.info.ast() == node) {
             if (import.info.type() == ImportType::Library
                     && !import.libraryPath.isEmpty()) {
-                QString msg = tr("Library at %1").arg(import.libraryPath);
+                QString msg = Tr::tr("Library at %1").arg(import.libraryPath.toUrlishString());
                 const LibraryInfo &libraryInfo = scopeChain.context()->snapshot().libraryInfo(import.libraryPath);
                 if (libraryInfo.pluginTypeInfoStatus() == LibraryInfo::DumpDone) {
                     msg += QLatin1Char('\n');
-                    msg += tr("Dumped plugins successfully.");
+                    msg += Tr::tr("Dumped plugins successfully.");
                 } else if (libraryInfo.pluginTypeInfoStatus() == LibraryInfo::TypeInfoFileDone) {
                     msg += QLatin1Char('\n');
-                    msg += tr("Read typeinfo files successfully.");
+                    msg += Tr::tr("Read typeinfo files successfully.");
                 }
                 setToolTip(msg);
             } else {
@@ -388,6 +381,13 @@ void QmlJSHoverHandler::reset()
 
 void QmlJSHoverHandler::operateTooltip(TextEditorWidget *editorWidget, const QPoint &point)
 {
+    // disable hoverhandling in case qmlls is enabled
+    if (editorWidget->textDocument()
+        && qmllsSettings()->isEnabledOnProjectFile(editorWidget->textDocument()->filePath())) {
+        BaseHoverHandler::operateTooltip(editorWidget, point);
+        return;
+    }
+
     if (toolTip().isEmpty())
         Utils::ToolTip::hide();
     else if (m_colorTip.isValid())
@@ -494,7 +494,10 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
                                                           + "::" + name,
                                                       "QML." + className + "::" + name,
                                                       className + "::" + name};
-                const HelpItem helpItem(helpIdCandidates, name, HelpItem::QmlProperty);
+                const HelpItem helpItem(helpIdCandidates,
+                                        qmlDocument->fileName(),
+                                        name,
+                                        HelpItem::QmlProperty);
                 if (helpItem.isValid()) {
                     setLastHelpItemIdentified(helpItem);
                     return true;
@@ -517,6 +520,5 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
     return false;
 }
 
-} // namespace Internal
 } // namespace QmlJSEditor
 
