@@ -40,6 +40,7 @@
 
 #include "application.h"
 #include "consoleprogressobserver.h"
+#include "session.h"
 #include "status.h"
 #include "parser/commandlineoption.h"
 #include "../shared/logging/consolelogger.h"
@@ -77,6 +78,7 @@ CommandLineFrontend::CommandLineFrontend(const CommandLineParser &parser, Settin
 CommandLineFrontend::~CommandLineFrontend()
 {
     m_cancelTimer->stop();
+    delete m_observer;
 }
 
 // Called from interrupt handler. Don't do anything non-trivial here.
@@ -95,9 +97,9 @@ void CommandLineFrontend::checkCancelStatus()
         m_cancelTimer->stop();
         if (m_resolveJobs.empty() && m_buildJobs.empty())
             std::exit(EXIT_FAILURE);
-        for (AbstractJob * const job : qAsConst(m_resolveJobs))
+        for (AbstractJob * const job : std::as_const(m_resolveJobs))
             job->cancel();
-        for (AbstractJob * const job : qAsConst(m_buildJobs))
+        for (AbstractJob * const job : std::as_const(m_buildJobs))
             job->cancel();
         break;
     case CancelStatusCanceling:
@@ -129,12 +131,16 @@ void CommandLineFrontend::start()
                 throw ErrorInfo(error);
             }
             break;
+        case SessionCommandType: {
+            startSession();
+            return;
+        }
         default:
             break;
         }
 
         if (m_parser.showVersion()) {
-            puts(QBS_VERSION);
+            std::puts(QBS_VERSION);
             qApp->exit(EXIT_SUCCESS);
             return;
         }
@@ -150,14 +156,15 @@ void CommandLineFrontend::start()
         params.setSettingsDirectory(m_settings->baseDirectory());
         params.setOverrideBuildGraphData(m_parser.command() == ResolveCommandType);
         params.setPropertyCheckingMode(ErrorHandlingMode::Strict);
+        params.setDeprecationWarningMode(m_parser.deprecationWarningMode());
         if (!m_parser.buildBeforeInstalling() || !m_parser.commandCanResolve())
             params.setRestoreBehavior(SetupProjectParameters::RestoreOnly);
         const auto buildConfigs = m_parser.buildConfigurations();
         for (const QVariantMap &buildConfig : buildConfigs) {
             QVariantMap userConfig = buildConfig;
-            const QString configurationKey = QLatin1String("qbs.configurationName");
-            const QString profileKey = QLatin1String("qbs.profile");
-            const QString installRootKey = QLatin1String("qbs.installRoot");
+            const QString configurationKey = QStringLiteral("qbs.configurationName");
+            const QString profileKey = QStringLiteral("qbs.profile");
+            const QString installRootKey = QStringLiteral("qbs.installRoot");
             QString installRoot = userConfig.value(installRootKey).toString();
             if (!installRoot.isEmpty() && QFileInfo(installRoot).isRelative()) {
                 installRoot.prepend(QLatin1Char('/')).prepend(QDir::currentPath());
@@ -176,6 +183,7 @@ void CommandLineFrontend::start()
             params.setConfigurationName(configurationName);
             params.setBuildRoot(buildDirectory(profileName));
             params.setOverriddenValues(userConfig);
+            params.setMaxJobCount(m_parser.jobCount(profileName));
             SetupProjectJob * const job = Project().setupProject(params,
                     ConsoleLogger::instance().logSink(), this);
             connectJob(job);
@@ -229,7 +237,7 @@ void CommandLineFrontend::handleJobFinished(bool success, AbstractJob *job)
                 return;
             }
             cancel();
-        } else if (SetupProjectJob * const setupJob = qobject_cast<SetupProjectJob *>(job)) {
+        } else if (const auto setupJob = qobject_cast<SetupProjectJob * const>(job)) {
             m_resolveJobs.removeOne(job);
             m_projects.push_back(setupJob->project());
             if (m_observer && resolvingMultipleProjects())
@@ -315,7 +323,7 @@ void CommandLineFrontend::handleProcessResultReport(const qbs::ProcessResult &re
 
     LogWriter w = result.success() ? qbsInfo() : qbsError();
     w << shellQuote(QDir::toNativeSeparators(result.executableFilePath()), result.arguments())
-      << (hasOutput ? QString::fromLatin1("\n") : QString())
+      << (hasOutput ? QStringLiteral("\n") : QString())
       << (result.stdOut().empty() ? QString() : result.stdOut().join(QLatin1Char('\n')));
     if (!result.stdErr().empty())
         w << result.stdErr().join(QLatin1Char('\n')) << MessageTag(QStringLiteral("stdErr"));
@@ -341,22 +349,32 @@ CommandLineFrontend::ProductMap CommandLineFrontend::productsToUse() const
     ProductMap products;
     QStringList productNames;
     const bool useAll = m_parser.products().empty();
-    for (const Project &project : qAsConst(m_projects)) {
+    for (const Project &project : std::as_const(m_projects)) {
         QList<ProductData> &productList = products[project];
         const ProjectData projectData = project.projectData();
-        const auto products = projectData.allProducts();
-        for (const ProductData &product : products) {
-            if (useAll || m_parser.products().contains(product.name())) {
+        for (const ProductData &product : projectData.allProducts()) {
+            productNames << product.fullDisplayName();
+            if (useAll || m_parser.products().contains(product.fullDisplayName())) {
                 productList.push_back(product);
-                productNames << product.name();
             }
         }
     }
 
     const auto parsedProductNames = m_parser.products();
     for (const QString &productName : parsedProductNames) {
-        if (!productNames.contains(productName))
-            throw ErrorInfo(Tr::tr("No such product '%1'.").arg(productName));
+        if (!productNames.contains(productName)) {
+            QString msg;
+            if (productNames.size() <= 10) {
+                productNames.sort();
+                const QString available = productNames.join(QLatin1String("', '"));
+                msg = Tr::tr("No such product '%1'. "
+                             "Available products: '%2'").arg(productName, available);
+            } else {
+                msg = Tr::tr("No such product '%1'. Use 'list-products' to see "
+                                  "all available products.").arg(productName);
+            }
+            throw ErrorInfo(msg);
+        }
     }
 
     return products;
@@ -406,6 +424,7 @@ void CommandLineFrontend::handleProjectsResolved()
         break;
     case HelpCommandType:
     case VersionCommandType:
+    case SessionCommandType:
         Q_ASSERT_X(false, Q_FUNC_INFO, "Impossible.");
     }
 }
@@ -413,7 +432,7 @@ void CommandLineFrontend::handleProjectsResolved()
 void CommandLineFrontend::makeClean()
 {
     if (m_parser.products().empty()) {
-        for (const Project &project : qAsConst(m_projects)) {
+        for (const Project &project : std::as_const(m_projects)) {
             m_buildJobs << project.cleanAllProducts(m_parser.cleanOptions(project.profile()), this);
         }
     } else {
@@ -471,9 +490,22 @@ QString CommandLineFrontend::buildDirectory(const QString &profileName) const
     }
 
     QString projectName(QFileInfo(m_parser.projectFilePath()).baseName());
+    QString originalBuildDir = buildDir;
     buildDir.replace(BuildDirectoryOption::magicProjectString(), projectName);
+    const QString buildDirPlaceHolderMsgTemplate = Tr::tr(
+        "You must provide the path to the project file when using build directory "
+        "placeholder '%1'.");
+    if (buildDir != originalBuildDir && projectName.isEmpty()) {
+        throw ErrorInfo(
+            buildDirPlaceHolderMsgTemplate.arg(BuildDirectoryOption::magicProjectString()));
+    }
     QString projectDir(QFileInfo(m_parser.projectFilePath()).path());
+    originalBuildDir = buildDir;
     buildDir.replace(BuildDirectoryOption::magicProjectDirString(), projectDir);
+    if (buildDir != originalBuildDir && projectDir.isEmpty()) {
+        throw ErrorInfo(
+            buildDirPlaceHolderMsgTemplate.arg(BuildDirectoryOption::magicProjectDirString()));
+    }
     if (!QFileInfo(buildDir).isAbsolute())
         buildDir = QDir::currentPath() + QLatin1Char('/') + buildDir;
     buildDir = QDir::cleanPath(buildDir);
@@ -485,7 +517,7 @@ void CommandLineFrontend::build()
     if (m_parser.products().empty()) {
         const Project::ProductSelection productSelection = m_parser.withNonDefaultProducts()
                 ? Project::ProductSelectionWithNonDefault : Project::ProductSelectionDefaultOnly;
-        for (const Project &project : qAsConst(m_projects))
+        for (const Project &project : std::as_const(m_projects))
             m_buildJobs << project.buildAllProducts(buildOptions(project), productSelection, this);
     } else {
         const ProductMap &products = productsToUse();
@@ -531,11 +563,13 @@ void CommandLineFrontend::checkGeneratorName()
 void CommandLineFrontend::generate()
 {
     QBS_CHECK(!!m_generator);
-    m_generator->generate(m_projects,
-                          m_parser.buildConfigurations(),
-                          m_parser.installOptions(QString()),
-                          m_parser.settingsDir(),
-                          ConsoleLogger::instance(m_settings));
+    const ErrorInfo error = m_generator->generate(m_projects,
+                                                  m_parser.buildConfigurations(),
+                                                  m_parser.installOptions(QString()),
+                                                  m_parser.settingsDir(),
+                                                  ConsoleLogger::instance(m_settings));
+    if (error.hasError())
+        throw error;
 }
 
 int CommandLineFrontend::runTarget()
@@ -544,7 +578,7 @@ int CommandLineFrontend::runTarget()
     const QString executableFilePath = productToRun.targetExecutable();
     if (executableFilePath.isEmpty()) {
         throw ErrorInfo(Tr::tr("Cannot run: Product '%1' is not an application.")
-                    .arg(productToRun.name()));
+                    .arg(productToRun.fullDisplayName()));
     }
     RunEnvironment runEnvironment = m_projects.front().getRunEnvironment(productToRun,
             m_parser.installOptions(m_projects.front().profile()),
@@ -573,17 +607,23 @@ void CommandLineFrontend::dumpNodesTree()
 
 void CommandLineFrontend::listProducts()
 {
-    const QList<ProductData> products = productsToUse().begin().value();
+    const QList<ProductData> products = productsToUse().constBegin().value();
     QStringList output;
-    for (const ProductData &p : products)
-        output += p.fullDisplayName();
+    for (const ProductData &p : products) {
+        QString productInfo = p.fullDisplayName();
+        if (!p.isEnabled())
+            productInfo.append(QLatin1Char(' ')).append(Tr::tr("[disabled]"));
+        else if (!p.properties().value(QStringLiteral("builtByDefault")).toBool())
+            productInfo.append(QLatin1Char(' ')).append(Tr::tr("[not built by default]"));
+        output += productInfo;
+    }
     output.sort();
     qbsInfo() << output.join(QLatin1Char('\n'));
 }
 
 void CommandLineFrontend::connectBuildJobs()
 {
-    for (AbstractJob * const job : qAsConst(m_buildJobs))
+    for (AbstractJob * const job : std::as_const(m_buildJobs))
         connectBuildJob(job);
 }
 
@@ -591,7 +631,7 @@ void CommandLineFrontend::connectBuildJob(AbstractJob *job)
 {
     connectJob(job);
 
-    BuildJob *bjob = qobject_cast<BuildJob *>(job);
+    const auto bjob = qobject_cast<const BuildJob *>(job);
     if (!bjob)
         return;
 
@@ -620,18 +660,16 @@ ProductData CommandLineFrontend::getTheOneRunnableProduct()
     QBS_CHECK(m_projects.size() == 1); // Has been checked earlier.
 
     if (m_parser.products().size() == 1) {
-        const auto products = m_projects.front().projectData().allProducts();
-        for (const ProductData &p : products) {
-            if (p.name() == m_parser.products().front())
+        for (const ProductData &p : m_projects.front().projectData().allProducts()) {
+            if (p.fullDisplayName() == m_parser.products().constFirst())
                 return p;
         }
         QBS_CHECK(false);
     }
-    QBS_CHECK(m_parser.products().size() == 0);
+    QBS_CHECK(m_parser.products().isEmpty());
 
     QList<ProductData> runnableProducts;
-    const auto products = m_projects.front().projectData().allProducts();
-    for (const ProductData &p : products) {
+    for (const ProductData &p : m_projects.front().projectData().allProducts()) {
         if (p.isRunnable())
             runnableProducts.push_back(p);
     }
@@ -647,14 +685,8 @@ ProductData CommandLineFrontend::getTheOneRunnableProduct()
     ErrorInfo error(Tr::tr("Ambiguous use of command '%1': No product given, but project "
                            "has more than one runnable product.").arg(m_parser.commandName()));
     error.append(Tr::tr("Use the '--products' option with one of the following products:"));
-    for (const ProductData &p : qAsConst(runnableProducts)) {
-        QString productRepr = QLatin1String("\t") + p.name();
-        if (p.profile() != m_projects.front().profile()) {
-            productRepr.append(QLatin1String(" [")).append(p.profile())
-                    .append(QLatin1Char(']'));
-        }
-        error.append(productRepr);
-    }
+    for (const ProductData &p : std::as_const(runnableProducts))
+        error.append(QLatin1String("\t") + p.fullDisplayName());
     throw error;
 }
 
