@@ -1,39 +1,48 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #include "commandlineparser.h"
 
-#include "command.h"
 #include "commandlineoption.h"
 #include "commandlineoptionpool.h"
 #include "commandpool.h"
+#include "parsercommand.h"
 #include "../qbstool.h"
 #include "../../shared/logging/consolelogger.h"
 
@@ -45,13 +54,19 @@
 #include <tools/hostosinfo.h>
 #include <tools/installoptions.h>
 #include <tools/preferences.h>
+#include <tools/qbsassert.h>
+#include <tools/qttools.h>
 #include <tools/settings.h>
+#include <tools/settingsrepresentation.h>
+#include <tools/stlutils.h>
 
-#include <QCoreApplication>
-#include <QDir>
-#include <QMap>
-#include <QPair>
-#include <QTextStream>
+#include <QtCore/qcoreapplication.h>
+#include <QtCore/qdir.h>
+#include <QtCore/qmap.h>
+#include <QtCore/qtextstream.h>
+
+#include <algorithm>
+#include <utility>
 
 #ifdef Q_OS_UNIX
 #include <unistd.h>
@@ -75,9 +90,9 @@ public:
     void setupProgress();
     void setupLogLevel();
     void setupBuildOptions();
+    void setupBuildConfigurations();
     bool checkForExistingBuildConfiguration(const QList<QVariantMap> &buildConfigs,
                                             const QString &configurationName);
-    bool isSameProfile(const QString &profile1, const QString &profile2) const;
     bool withNonDefaultProducts() const;
     bool dryRun() const;
     QString settingsDir() const { return  optionPool.settingsDirOption()->settingsDir(); }
@@ -91,29 +106,25 @@ public:
     QString projectFilePath;
     QString projectBuildDirectory;
     BuildOptions buildOptions;
+    QList<QVariantMap> buildConfigurations;
     CommandLineOptionPool optionPool;
     CommandPool commandPool;
     bool showProgress;
     bool logTime;
 };
 
-CommandLineParser::CommandLineParser() : d(0)
-{
-}
+CommandLineParser::CommandLineParser() = default;
 
-CommandLineParser::~CommandLineParser()
-{
-    delete d;
-}
+CommandLineParser::~CommandLineParser() = default;
 
 void CommandLineParser::printHelp() const
 {
     QTextStream stream(stdout);
 
     Q_ASSERT(d->command == d->commandPool.getCommand(HelpCommandType));
-    const HelpCommand * const helpCommand = static_cast<HelpCommand *>(d->command);
+    const auto helpCommand = static_cast<const HelpCommand *>(d->command);
     if (helpCommand->commandToDescribe().isEmpty()) {
-        stream << "qbs " QBS_VERSION "\n";
+        stream << "Qbs " QBS_VERSION ", a cross-platform build tool.\n";
         stream << d->generalHelp();
     } else {
         const Command * const commandToDescribe
@@ -121,7 +132,7 @@ void CommandLineParser::printHelp() const
         if (commandToDescribe) {
             stream << commandToDescribe->longDescription();
         } else if (!QbsTool::tryToRunTool(helpCommand->commandToDescribe(),
-                                          QStringList(QLatin1String("--help")))) {
+                                          QStringList(QStringLiteral("--help")))) {
             throw ErrorInfo(Tr::tr("No such command '%1'.\n%2")
                         .arg(helpCommand->commandToDescribe(), d->generalHelp()));
         }
@@ -145,23 +156,16 @@ QString CommandLineParser::projectBuildDirectory() const
 
 BuildOptions CommandLineParser::buildOptions(const QString &profile) const
 {
-    Settings settings(settingsDir());
-    Preferences preferences(&settings, profile);
-
-    if (d->buildOptions.maxJobCount() <= 0) {
-        d->buildOptions.setMaxJobCount(preferences.jobs());
-    }
-
+    d->buildOptions.setMaxJobCount(jobCount(profile));
     if (d->buildOptions.echoMode() < 0) {
-        d->buildOptions.setEchoMode(preferences.defaultEchoMode());
+        Settings settings(settingsDir());
+        d->buildOptions.setEchoMode(Preferences(&settings, profile).defaultEchoMode());
     }
-
     return d->buildOptions;
 }
 
 CleanOptions CommandLineParser::cleanOptions(const QString &profile) const
 {
-    Q_ASSERT(command() == CleanCommandType);
     CleanOptions options;
     options.setDryRun(buildOptions(profile).dryRun());
     options.setKeepGoing(buildOptions(profile).keepGoing());
@@ -171,7 +175,6 @@ CleanOptions CommandLineParser::cleanOptions(const QString &profile) const
 
 GenerateOptions CommandLineParser::generateOptions() const
 {
-    Q_ASSERT(command() == GenerateCommandType);
     GenerateOptions options;
     options.setGeneratorName(d->optionPool.generatorOption()->generatorName());
     return options;
@@ -179,8 +182,6 @@ GenerateOptions CommandLineParser::generateOptions() const
 
 InstallOptions CommandLineParser::installOptions(const QString &profile) const
 {
-    Q_ASSERT(command() == InstallCommandType || command() == RunCommandType
-             || command() == GenerateCommandType);
     InstallOptions options;
     options.setRemoveExistingInstallation(d->optionPool.removeFirstoption()->enabled());
     options.setInstallRoot(d->optionPool.installRootOption()->installRoot());
@@ -196,9 +197,13 @@ InstallOptions CommandLineParser::installOptions(const QString &profile) const
     return options;
 }
 
-bool CommandLineParser::force() const
+int CommandLineParser::jobCount(const QString &profile) const
 {
-    return d->optionPool.forceOption()->enabled();
+    if (const int explicitJobCount = d->optionPool.jobsOption()->jobCount(); explicitJobCount > 0)
+        return explicitJobCount;
+
+    Settings settings(settingsDir());
+    return Preferences(&settings, profile).jobs();
 }
 
 bool CommandLineParser::forceTimestampCheck() const
@@ -252,6 +257,11 @@ QStringList CommandLineParser::products() const
     return d->optionPool.productsOption()->arguments();
 }
 
+QStringList CommandLineParser::runEnvConfig() const
+{
+    return d->optionPool.runEnvConfigOption()->arguments();
+}
+
 bool CommandLineParser::showProgress() const
 {
     return d->showProgress;
@@ -259,7 +269,7 @@ bool CommandLineParser::showProgress() const
 
 bool CommandLineParser::showVersion() const
 {
-    return d->optionPool.versionOption()->enabled();
+    return d->command->type() == VersionCommandType;
 }
 
 QString CommandLineParser::settingsDir() const
@@ -267,11 +277,21 @@ QString CommandLineParser::settingsDir() const
     return d->settingsDir();
 }
 
+DeprecationWarningMode CommandLineParser::deprecationWarningMode() const
+{
+    return d->optionPool.deprecationWarningsOption()->mode();
+}
+
 QString CommandLineParser::commandName() const
 {
     return d->command->representation();
 }
 
+bool CommandLineParser::commandCanResolve() const
+{
+    return d->command->canResolve();
+}
+
 QString CommandLineParser::commandDescription() const
 {
     return d->command->longDescription();
@@ -279,70 +299,17 @@ QString CommandLineParser::commandDescription() const
 
 static QString getBuildConfigurationName(const QVariantMap &buildConfig)
 {
-    return buildConfig.value(QLatin1String("qbs.configurationName")).toString();
+    return buildConfig.value(QStringLiteral("qbs.configurationName")).toString();
 }
 
 QList<QVariantMap> CommandLineParser::buildConfigurations() const
 {
-    // first: configuration name, second: properties.
-    // Empty configuration name used for global properties.
-    typedef QPair<QString, QVariantMap> PropertyListItem;
-    QList<PropertyListItem> propertiesPerConfiguration;
-
-    const QString configurationNameKey = QLatin1String("qbs.configurationName");
-    QString currentConfigurationName;
-    QVariantMap currentProperties;
-    foreach (const QString &arg, d->command->additionalArguments()) {
-        const int sepPos = arg.indexOf(QLatin1Char(':'));
-        if (sepPos == -1) { // New build configuration found.
-            propertiesPerConfiguration << qMakePair(currentConfigurationName, currentProperties);
-            currentConfigurationName = arg;
-            currentProperties.clear();
-            continue;
-        }
-        const QString property = d->propertyName(arg.left(sepPos));
-        if (property.isEmpty()) {
-            qbsWarning() << Tr::tr("Ignoring empty property.");
-        } else if (property == configurationNameKey) {
-            qbsWarning() << Tr::tr("Refusing to overwrite special property '%1'.")
-                            .arg(configurationNameKey);
-        } else {
-            currentProperties.insert(property, arg.mid(sepPos + 1));
-        }
-    }
-    propertiesPerConfiguration << qMakePair(currentConfigurationName, currentProperties);
-
-    if (propertiesPerConfiguration.count() == 1) // No configuration name specified on command line.
-        propertiesPerConfiguration << PropertyListItem(QStringLiteral("default"), QVariantMap());
-
-    const QVariantMap globalProperties = propertiesPerConfiguration.takeFirst().second;
-    QList<QVariantMap> buildConfigs;
-    foreach (const PropertyListItem &item, propertiesPerConfiguration) {
-        QVariantMap properties = item.second;
-        for (QVariantMap::ConstIterator globalPropIt = globalProperties.constBegin();
-                 globalPropIt != globalProperties.constEnd(); ++globalPropIt) {
-            if (!properties.contains(globalPropIt.key()))
-                properties.insert(globalPropIt.key(), globalPropIt.value());
-        }
-
-        const QString configurationName = item.first;
-        if (d->checkForExistingBuildConfiguration(buildConfigs, configurationName)) {
-            qbsWarning() << Tr::tr("Ignoring redundant request to build for configuration '%1'.")
-                            .arg(configurationName);
-            continue;
-        }
-
-        properties.insert(configurationNameKey, configurationName);
-        buildConfigs << properties;
-    }
-
-    return buildConfigs;
+    return d->buildConfigurations;
 }
 
 bool CommandLineParser::parseCommandLine(const QStringList &args)
 {
-    delete d;
-    d = new CommandLineParserPrivate;
+    d = std::make_unique<CommandLineParserPrivate>();
     d->commandLine = args;
     try {
         d->doParse();
@@ -355,25 +322,39 @@ bool CommandLineParser::parseCommandLine(const QStringList &args)
 
 
 CommandLineParser::CommandLineParserPrivate::CommandLineParserPrivate()
-    : command(0), commandPool(optionPool), showProgress(false), logTime(false)
+    : command(nullptr), commandPool(optionPool), showProgress(false), logTime(false)
 {
 }
 
 void CommandLineParser::CommandLineParserPrivate::doParse()
 {
-    if (commandLine.isEmpty()) { // No command given, use default.
+    if (commandLine.empty()) { // No command given, use default.
         command = commandPool.getCommand(BuildCommandType);
     } else {
-        command = commandFromString(commandLine.first());
+        command = commandFromString(commandLine.front());
         if (command) {
-            commandLine.removeFirst();
-        } else { // No command given.
-            // As an exception to the command-based syntax, we allow -h or --help as the
-            // sole contents of the command line, because people are used to this working.
-            if (commandLine.count() == 1 && (commandLine.first() == QLatin1String("-h")
-                                             || commandLine.first() == QLatin1String("--help"))) {
+            const QString commandName = commandLine.takeFirst();
+
+            // if the command line contains a `<command>` with
+            // either `-h` or `--help` switch, we transform
+            // it to corresponding `help <command>` instead
+            const QStringList helpSwitches = {QStringLiteral("-h"), QStringLiteral("--help")};
+            if (auto it = std::find_first_of(
+                    commandLine.begin(), commandLine.end(),
+                    helpSwitches.begin(), helpSwitches.end());
+                    it != commandLine.end()) {
                 command = commandPool.getCommand(HelpCommandType);
-                commandLine.clear();
+                commandLine = QList{commandName}; // keep only command's name
+            }
+        } else { // No command given.
+            if (commandLine.front() == QLatin1String("-h")
+                    || commandLine.front() == QLatin1String("--help")) {
+                command = commandPool.getCommand(HelpCommandType);
+                commandLine.takeFirst();
+            } else if (commandLine.front() == QLatin1String("-V")
+                       || commandLine.front() == QLatin1String("--version")) {
+                command = commandPool.getCommand(VersionCommandType);
+                commandLine.takeFirst();
             } else {
                 command = commandPool.getCommand(BuildCommandType);
             }
@@ -381,14 +362,12 @@ void CommandLineParser::CommandLineParserPrivate::doParse()
     }
     command->parse(commandLine);
 
-    if (command->type() == HelpCommandType)
+    if (command->type() == HelpCommandType || command->type() == VersionCommandType)
         return;
 
-    if (command->type() == BuildCommandType && optionPool.versionOption()->enabled())
-        return;
-
-    setupProjectFile();
     setupBuildDirectory();
+    setupBuildConfigurations();
+    setupProjectFile();
     setupProgress();
     setupLogLevel();
     setupBuildOptions();
@@ -396,61 +375,75 @@ void CommandLineParser::CommandLineParserPrivate::doParse()
 
 Command *CommandLineParser::CommandLineParserPrivate::commandFromString(const QString &commandString) const
 {
-    foreach (Command * const command, allCommands()) {
+    const auto commands = allCommands();
+    for (Command * const command : commands) {
         if (command->representation() == commandString)
             return command;
     }
-    return 0;
+    return nullptr;
 }
 
 QList<Command *> CommandLineParser::CommandLineParserPrivate::allCommands() const
 {
-    return QList<Command *>()
-            << commandPool.getCommand(GenerateCommandType)
-            << commandPool.getCommand(ResolveCommandType)
-            << commandPool.getCommand(BuildCommandType)
-            << commandPool.getCommand(CleanCommandType)
-            << commandPool.getCommand(RunCommandType)
-            << commandPool.getCommand(ShellCommandType)
-            << commandPool.getCommand(StatusCommandType)
-            << commandPool.getCommand(UpdateTimestampsCommandType)
-            << commandPool.getCommand(InstallCommandType)
-            << commandPool.getCommand(DumpNodesTreeCommandType)
-            << commandPool.getCommand(HelpCommandType);
+    return {commandPool.getCommand(GenerateCommandType),
+            commandPool.getCommand(ResolveCommandType),
+            commandPool.getCommand(BuildCommandType),
+            commandPool.getCommand(CleanCommandType),
+            commandPool.getCommand(RunCommandType),
+            commandPool.getCommand(ShellCommandType),
+            commandPool.getCommand(StatusCommandType),
+            commandPool.getCommand(UpdateTimestampsCommandType),
+            commandPool.getCommand(InstallCommandType),
+            commandPool.getCommand(DumpNodesTreeCommandType),
+            commandPool.getCommand(ListProductsCommandType),
+            commandPool.getCommand(VersionCommandType),
+            commandPool.getCommand(SessionCommandType),
+            commandPool.getCommand(HelpCommandType)};
+}
+
+static QString extractToolDescription(const QString &tool, const QString &output)
+{
+    if (tool == QLatin1String("create-project")) {
+        // This command uses QCommandLineParser, where the description is not in the first line.
+        const int eol1Pos = output.indexOf(QLatin1Char('\n'));
+        const int eol2Pos = output.indexOf(QLatin1Char('\n'), eol1Pos + 1);
+        return output.mid(eol1Pos + 1, eol2Pos - eol1Pos - 1);
+    }
+    return output.left(output.indexOf(QLatin1Char('\n')));
 }
 
 QString CommandLineParser::CommandLineParserPrivate::generalHelp() const
 {
     QString help = Tr::tr("Usage: qbs [command] [command parameters]\n");
-    help += Tr::tr("Internal commands:\n");
+    help += Tr::tr("Built-in commands:\n");
     const int rhsIndentation = 30;
 
     // Sorting the commands by name is nicer for the user.
     QMap<QString, const Command *> commandMap;
-    foreach (const Command * command, allCommands())
+    const auto commands = allCommands();
+    for (const Command * command : commands)
         commandMap.insert(command->representation(), command);
 
-    foreach (const Command * command, commandMap) {
+    for (const Command * command : std::as_const(commandMap)) {
         help.append(QLatin1String("  ")).append(command->representation());
         const QString whitespace
-                = QString(rhsIndentation - 2 - command->representation().count(), QLatin1Char(' '));
+                = QString(rhsIndentation - 2 - command->representation().size(), QLatin1Char(' '));
         help.append(whitespace).append(command->shortDescription()).append(QLatin1Char('\n'));
     }
 
     QStringList toolNames = QbsTool::allToolNames();
     toolNames.sort();
-    if (!toolNames.isEmpty()) {
+    if (!toolNames.empty()) {
         help.append(QLatin1Char('\n')).append(Tr::tr("Auxiliary commands:\n"));
-        foreach (const QString &toolName, toolNames) {
+        for (const QString &toolName : std::as_const(toolNames)) {
             help.append(QLatin1String("  ")).append(toolName);
-            const QString whitespace = QString(rhsIndentation - 2 - toolName.count(),
+            const QString whitespace = QString(rhsIndentation - 2 - toolName.size(),
                                                QLatin1Char(' '));
             QbsTool tool;
-            tool.runTool(toolName, QStringList(QLatin1String("--help")));
+            tool.runTool(toolName, QStringList(QStringLiteral("--help")));
             if (tool.exitCode() != 0)
                 continue;
-            const QString shortDescription
-                    = tool.stdOut().left(tool.stdOut().indexOf(QLatin1Char('\n')));
+            const QString shortDescription = extractToolDescription(toolName, tool.stdOut());
             help.append(whitespace).append(shortDescription).append(QLatin1Char('\n'));
         }
     }
@@ -461,44 +454,6 @@ QString CommandLineParser::CommandLineParserPrivate::generalHelp() const
 void CommandLineParser::CommandLineParserPrivate::setupProjectFile()
 {
     projectFilePath = optionPool.fileOption()->projectFilePath();
-    if (projectFilePath.isEmpty()) {
-        qbsDebug() << "No project file given; looking in current directory.";
-        projectFilePath = QDir::currentPath();
-    }
-
-    const QFileInfo projectFileInfo(projectFilePath);
-    if (!projectFileInfo.exists())
-        throw ErrorInfo(Tr::tr("Project file '%1' cannot be found.").arg(projectFilePath));
-    if (projectFileInfo.isRelative())
-        projectFilePath = projectFileInfo.absoluteFilePath();
-    if (projectFileInfo.isFile())
-        return;
-    if (!projectFileInfo.isDir())
-        throw ErrorInfo(Tr::tr("Project file '%1' has invalid type.").arg(projectFilePath));
-
-    const QStringList namePatterns = QStringList()
-            << QLatin1String("*.qbs");
-
-    const QStringList &actualFileNames
-            = QDir(projectFilePath).entryList(namePatterns, QDir::Files);
-    if (actualFileNames.isEmpty()) {
-        QString error;
-        if (optionPool.fileOption()->projectFilePath().isEmpty())
-            error = Tr::tr("No project file given and none found in current directory.\n");
-        else
-            error = Tr::tr("No project file found in directory '%1'.").arg(projectFilePath);
-        throw ErrorInfo(error);
-    }
-    if (actualFileNames.count() > 1) {
-        throw ErrorInfo(Tr::tr("More than one project file found in directory '%1'.")
-                .arg(projectFilePath));
-    }
-    projectFilePath.append(QLatin1Char('/')).append(actualFileNames.first());
-
-    projectFilePath = QDir::current().filePath(projectFilePath);
-    projectFilePath = QDir::cleanPath(projectFilePath);
-
-    qbsDebug() << "Using project file '" << QDir::toNativeSeparators(projectFilePath) << "'.";
 }
 
 void CommandLineParser::CommandLineParserPrivate::setupBuildDirectory()
@@ -511,10 +466,8 @@ void CommandLineParser::CommandLineParserPrivate::setupBuildOptions()
     buildOptions.setDryRun(dryRun());
     QStringList changedFiles = optionPool.changedFilesOption()->arguments();
     QDir currentDir;
-    for (int i = 0; i < changedFiles.count(); ++i) {
-        QString &file = changedFiles[i];
+    for (QString &file : changedFiles)
         file = QDir::fromNativeSeparators(currentDir.absoluteFilePath(file));
-    }
     buildOptions.setChangedFiles(changedFiles);
     buildOptions.setKeepGoing(optionPool.keepGoingOption()->enabled());
     buildOptions.setForceTimestampCheck(optionPool.forceTimestampCheckOption()->enabled());
@@ -525,6 +478,66 @@ void CommandLineParser::CommandLineParserPrivate::setupBuildOptions()
     buildOptions.setEchoMode(echoMode());
     buildOptions.setInstall(!optionPool.noInstallOption()->enabled());
     buildOptions.setRemoveExistingInstallation(optionPool.removeFirstoption()->enabled());
+    buildOptions.setJobLimits(optionPool.jobLimitsOption()->jobLimits());
+    buildOptions.setProjectJobLimitsTakePrecedence(
+                optionPool.respectProjectJobLimitsOption()->enabled());
+    buildOptions.setSettingsDirectory(settingsDir());
+}
+
+void CommandLineParser::CommandLineParserPrivate::setupBuildConfigurations()
+{
+    // first: configuration name, second: properties.
+    // Empty configuration name used for global properties.
+    using PropertyListItem = std::pair<QString, QVariantMap>;
+    QList<PropertyListItem> propertiesPerConfiguration;
+
+    const QString configurationNameKey = QStringLiteral("qbs.configurationName");
+    QString currentConfigurationName;
+    QVariantMap currentProperties;
+    const auto args = command->additionalArguments();
+    for (const QString &arg : args) {
+        const int sepPos = arg.indexOf(QLatin1Char(':'));
+        QBS_CHECK(sepPos > 0);
+        const QString key = arg.left(sepPos);
+        const QString rawValue = arg.mid(sepPos + 1);
+        if (key == QLatin1String("config") || key == configurationNameKey) {
+            propertiesPerConfiguration.push_back(std::make_pair(currentConfigurationName,
+                                                                currentProperties));
+            currentConfigurationName = rawValue;
+            currentProperties.clear();
+            continue;
+        }
+        currentProperties.insert(propertyName(key), representationToSettingsValue(rawValue));
+    }
+    propertiesPerConfiguration.push_back(std::make_pair(currentConfigurationName,
+                                                        currentProperties));
+
+    if (propertiesPerConfiguration.size() == 1) // No configuration name specified on command line.
+        propertiesPerConfiguration.push_back(PropertyListItem(QStringLiteral("default"),
+                                                              QVariantMap()));
+
+    const QVariantMap globalProperties = propertiesPerConfiguration.takeFirst().second;
+    QList<QVariantMap> buildConfigs;
+    for (const PropertyListItem &item : std::as_const(propertiesPerConfiguration)) {
+        QVariantMap properties = item.second;
+        for (QVariantMap::ConstIterator globalPropIt = globalProperties.constBegin();
+                 globalPropIt != globalProperties.constEnd(); ++globalPropIt) {
+            if (!properties.contains(globalPropIt.key()))
+                properties.insert(globalPropIt.key(), globalPropIt.value());
+        }
+
+        const QString configurationName = item.first;
+        if (checkForExistingBuildConfiguration(buildConfigs, configurationName)) {
+            qbsWarning() << Tr::tr("Ignoring redundant request to build for configuration '%1'.")
+                            .arg(configurationName);
+            continue;
+        }
+
+        properties.insert(configurationNameKey, configurationName);
+        buildConfigs.push_back(properties);
+    }
+
+    buildConfigurations = buildConfigs;
 }
 
 void CommandLineParser::CommandLineParserPrivate::setupProgress()
@@ -586,31 +599,15 @@ QString CommandLineParser::CommandLineParserPrivate::propertyName(const QString
     // Make fully-qualified, ie "platform" -> "qbs.platform"
     if (aCommandLineName.contains(QLatin1Char('.')))
         return aCommandLineName;
-    else
-        return QLatin1String("qbs.") + aCommandLineName;
+    return QLatin1String("qbs.") + aCommandLineName;
 }
 
 bool CommandLineParser::CommandLineParserPrivate::checkForExistingBuildConfiguration(
         const QList<QVariantMap> &buildConfigs, const QString &configurationName)
 {
-    foreach (const QVariantMap &buildConfig, buildConfigs) {
-        if (configurationName == getBuildConfigurationName(buildConfig))
-            return true;
-    }
-    return false;
-}
-
-bool CommandLineParser::CommandLineParserPrivate::isSameProfile(const QString &profile1,
-                                                                const QString &profile2) const
-{
-    if (profile1 == profile2)
-        return true;
-    Settings settings(settingsDir());
-    if (profile1.isEmpty())
-        return profile2.isEmpty() || profile2 == settings.defaultProfile();
-    if (profile2.isEmpty())
-        return profile1 == settings.defaultProfile();
-    return false;
+    return Internal::any_of(buildConfigs, [&configurationName](const auto &buildConfig) {
+        return configurationName == getBuildConfigurationName(buildConfig);
+    });
 }
 
 bool CommandLineParser::CommandLineParserPrivate::withNonDefaultProducts() const
@@ -622,7 +619,7 @@ bool CommandLineParser::CommandLineParserPrivate::withNonDefaultProducts() const
 
 bool CommandLineParser::CommandLineParserPrivate::dryRun() const
 {
-     if (command->type() == GenerateCommandType)
+     if (command->type() == GenerateCommandType || command->type() == ListProductsCommandType)
          return true;
      return optionPool.dryRunOption()->enabled();
 }
@@ -632,7 +629,7 @@ CommandEchoMode CommandLineParser::CommandLineParserPrivate::echoMode() const
     if (command->type() == GenerateCommandType)
         return CommandEchoModeSilent;
 
-    if (optionPool.commandEchoModeOption()->commandEchoMode() >= 0)
+    if (optionPool.commandEchoModeOption()->commandEchoMode() < CommandEchoModeInvalid)
         return optionPool.commandEchoModeOption()->commandEchoMode();
 
     return defaultCommandEchoMode();
